-module(megaco_text_parser_v2).
-define(THIS_MODULE, megaco_text_parser_v2).
-export([parse/1, parse_and_scan/1, format_error/1]).

%% The following directive is needed for (significantly) faster compilation
%% of the generated .erl file by the HiPE compiler.  Please do not remove.
-compile([{hipe,[{regalloc,linear_scan}]}]).

-include("megaco_text_parser_v2.hrl").



%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.1, (the "License"); you may not use this file except in
%% compliance with the License. You should have received a copy of the
%% Erlang Public License along with this software. If not, it can be
%% retrieved via the world wide web at http://www.erlang.org/.
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Initial Developer of the Original Code is Ericsson Utvecklings AB.
%% Portions created by Ericsson are Copyright 1999, Ericsson Utvecklings
%% AB. All Rights Reserved.''
%% 
%%     $Id$
%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The parser generator will insert appropriate declarations before this line.%

parse(Tokens) ->
    case catch yeccpars1(Tokens, false, 0, [], []) of
	error ->
	    Errorline =
		if Tokens == [] -> 0; true -> element(2, hd(Tokens)) end,
	    {error,
	     {Errorline, ?THIS_MODULE, "syntax error at or after this line."}};
	Other ->
	    Other
    end.

parse_and_scan({Mod, Fun, Args}) ->
    case apply(Mod, Fun, Args) of
	{eof, _} ->
	    {ok, eof};
	{error, Descriptor, _} ->
	    {error, Descriptor};
	{ok, Tokens, _} ->
	    yeccpars1(Tokens, {Mod, Fun, Args}, 0, [], [])
    end.

format_error(Message) ->
    case io_lib:deep_char_list(Message) of
	true ->
	    Message;
	_ ->
	    io_lib:write(Message)
    end.

% To be used in grammar files to throw an error message to the parser toplevel.
% Doesn't have to be exported!
return_error(Line, Message) ->
    throw({error, {Line, ?THIS_MODULE, Message}}).


% Don't change yeccpars1/6 too much, it is called recursively by yeccpars2/8!
yeccpars1([Token | Tokens], Tokenizer, State, States, Vstack) ->
    yeccpars2(State, element(1, Token), States, Vstack, Token, Tokens,
	      Tokenizer);
yeccpars1([], {M, F, A}, State, States, Vstack) ->
    case catch apply(M, F, A) of
        {eof, _Endline} ->
            yeccpars1([], false, State, States, Vstack);
        {error, Descriptor, _Endline} ->
            {error, Descriptor};
        {'EXIT', Reason} ->
            {error, {0, ?THIS_MODULE, Reason}};
        {ok, Tokens, _Endline} ->
	    case catch yeccpars1(Tokens, {M, F, A}, State, States, Vstack) of
		error ->
		    Errorline = element(2, hd(Tokens)),
		    {error, {Errorline, ?THIS_MODULE,
			     "syntax error at or after this line."}};
		Other ->
		    Other
	    end
    end;
yeccpars1([], false, State, States, Vstack) ->
    yeccpars2(State, '$end', States, Vstack, {'$end', 999999}, [], false).

% For internal use only.
yeccerror(Token) ->
    {error,
     {element(2, Token), ?THIS_MODULE,
      ["syntax error before: ", yecctoken2string(Token)]}}.

yecctoken2string({atom, _, A}) -> io_lib:write(A);
yecctoken2string({integer,_,N}) -> io_lib:write(N);
yecctoken2string({float,_,F}) -> io_lib:write(F);
yecctoken2string({char,_,C}) -> io_lib:write_char(C);
yecctoken2string({var,_,V}) -> io_lib:format('~s', [V]);
yecctoken2string({string,_,S}) -> io_lib:write_string(S);
yecctoken2string({reserved_symbol, _, A}) -> io_lib:format('~w', [A]);
yecctoken2string({_Cat, _, Val}) -> io_lib:format('~w', [Val]);

yecctoken2string({'dot', _}) -> io_lib:format('~w', ['.']);
yecctoken2string({'$end', _}) ->
    [];
yecctoken2string({Other, _}) when atom(Other) ->
    io_lib:format('~w', [Other]);
yecctoken2string(Other) ->
    io_lib:write(Other).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


yeccpars2(0, 'SEP', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 1, [0 | __Ss], [__T | __Stack]);
yeccpars2(0, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  no_sep,
 yeccpars2(3, __Cat, [0 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(1, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  sep,
 yeccpars2(yeccgoto(optSep, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(2, '$end', _, __Stack, _, _, _) ->
 {ok, hd(__Stack)};
yeccpars2(2, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(3, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 4, [3 | __Ss], [__T | __Stack]);
yeccpars2(3, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  asn1_NOVALUE,
 yeccpars2(5, __Cat, [3 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(4, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 861, [4 | __Ss], [__T | __Stack]);
yeccpars2(4, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(5, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [5 | __Ss], [__T | __Stack]);
yeccpars2(5, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(6, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(7, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(8, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(9, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(10, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(11, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(12, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(13, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(14, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(15, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(16, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(17, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(18, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(19, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(20, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(21, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(22, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(23, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(24, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(25, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(26, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(27, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(28, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(29, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(30, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(31, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(32, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(33, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(34, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(35, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(36, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(37, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(38, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(39, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(40, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(41, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(42, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(43, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(44, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(45, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(46, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(47, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(48, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(49, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(50, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(51, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(52, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(53, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(54, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(55, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(56, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(57, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(58, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(59, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(60, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(61, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(62, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(63, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(64, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(65, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(66, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(67, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(68, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(69, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(70, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(71, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(72, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(73, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(74, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(75, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(76, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(77, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(78, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(79, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(80, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(81, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(82, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(83, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(84, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(85, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(86, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(87, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(88, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(89, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(90, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(91, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(92, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(93, 'endOfMessage', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 860, [93 | __Ss], [__T | __Stack]);
yeccpars2(93, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(94, 'LSBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [94 | __Ss], [__T | __Stack]);
yeccpars2(94, 'LESSER', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [94 | __Ss], [__T | __Stack]);
yeccpars2(94, 'SEP', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 1, [94 | __Ss], [__T | __Stack]);
yeccpars2(94, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  no_sep,
 yeccpars2(100, __Cat, [94 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(95, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [95 | __Ss], [__T | __Stack]);
yeccpars2(95, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(96, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, 'COLON', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 846, [96 | __Ss], [__T | __Stack]);
yeccpars2(96, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(847, __Cat, [96 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(97, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(mId, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(98, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(mId, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(99, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 108, [99 | __Ss], [__T | __Stack]);
yeccpars2(99, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 110, [99 | __Ss], [__T | __Stack]);
yeccpars2(99, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 112, [99 | __Ss], [__T | __Stack]);
yeccpars2(99, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 109, [99 | __Ss], [__T | __Stack]);
yeccpars2(99, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 111, [99 | __Ss], [__T | __Stack]);
yeccpars2(99, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(100, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, 'MtpAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 101, [100 | __Ss], [__T | __Stack]);
yeccpars2(100, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(101, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_mtpAddress(__1),
 yeccpars2(yeccgoto(mtpAddress, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(102, 'SEP', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 1, [102 | __Ss], [__T | __Stack]);
yeccpars2(102, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  no_sep,
 yeccpars2(107, __Cat, [102 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(103, 'SEP', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 1, [103 | __Ss], [__T | __Stack]);
yeccpars2(103, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  no_sep,
 yeccpars2(106, __Cat, [103 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(104, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {deviceName,__1},
 yeccpars2(yeccgoto(deviceName, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(105, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_pathName(__1),
 yeccpars2(yeccgoto(pathName, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(106, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __2,
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(mId, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(107, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __2,
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(mId, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(108, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 830, [108 | __Ss], [__T | __Stack]);
yeccpars2(108, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(109, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 842, [109 | __Ss], [__T | __Stack]);
yeccpars2(109, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(110, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 700, [110 | __Ss], [__T | __Stack]);
yeccpars2(110, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(111, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 692, [111 | __Ss], [__T | __Stack]);
yeccpars2(111, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(112, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 122, [112 | __Ss], [__T | __Stack]);
yeccpars2(112, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 123, [112 | __Ss], [__T | __Stack]);
yeccpars2(112, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(113, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {messageError,__1},
 yeccpars2(yeccgoto(messageBody, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(114, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_message(__1,__2,__3),
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(message, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(115, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 110, [115 | __Ss], [__T | __Stack]);
yeccpars2(115, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 112, [115 | __Ss], [__T | __Stack]);
yeccpars2(115, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 109, [115 | __Ss], [__T | __Stack]);
yeccpars2(115, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 111, [115 | __Ss], [__T | __Stack]);
yeccpars2(115, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__1],
 yeccpars2(yeccgoto(transactionList, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(116, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {transactions,__1},
 yeccpars2(yeccgoto(messageBody, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(117, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {transactionPending,__1},
 yeccpars2(yeccgoto(transactionItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(118, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {transactionReply,__1},
 yeccpars2(yeccgoto(transactionItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(119, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {transactionRequest,__1},
 yeccpars2(yeccgoto(transactionItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(120, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {transactionResponseAck,__1},
 yeccpars2(yeccgoto(transactionItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(121, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__1|__2],
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(transactionList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(122, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 682, [122 | __Ss], [__T | __Stack]);
yeccpars2(122, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(123, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 124, [123 | __Ss], [__T | __Stack]);
yeccpars2(123, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(124, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 131, [124 | __Ss], [__T | __Stack]);
yeccpars2(124, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(125, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 126, [125 | __Ss], [__T | __Stack]);
yeccpars2(125, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(127, __Cat, [125 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(126, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 124, [126 | __Ss], [__T | __Stack]);
yeccpars2(126, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(127, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 128, [127 | __Ss], [__T | __Stack]);
yeccpars2(127, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(128, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'TransactionRequest'{transactionId = asn1_NOVALUE, actions = [__3|__4]},
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(transactionRequest, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(129, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 126, [129 | __Ss], [__T | __Stack]);
yeccpars2(129, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(130, __Cat, [129 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(130, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(actionRequestList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(131, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [131 | __Ss], [__T | __Stack]);
yeccpars2(131, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(132, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 134, [132 | __Ss], [__T | __Stack]);
yeccpars2(132, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(133, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_contextID(__1),
 yeccpars2(yeccgoto(contextID, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(134, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 144, [134 | __Ss], [__T | __Stack]);
yeccpars2(134, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 147, [134 | __Ss], [__T | __Stack]);
yeccpars2(134, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 145, [134 | __Ss], [__T | __Stack]);
yeccpars2(134, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 143, [134 | __Ss], [__T | __Stack]);
yeccpars2(134, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 136, [134 | __Ss], [__T | __Stack]);
yeccpars2(134, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 137, [134 | __Ss], [__T | __Stack]);
yeccpars2(134, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 146, [134 | __Ss], [__T | __Stack]);
yeccpars2(134, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 141, [134 | __Ss], [__T | __Stack]);
yeccpars2(134, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 142, [134 | __Ss], [__T | __Stack]);
yeccpars2(134, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 135, [134 | __Ss], [__T | __Stack]);
yeccpars2(134, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 138, [134 | __Ss], [__T | __Stack]);
yeccpars2(134, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 139, [134 | __Ss], [__T | __Stack]);
yeccpars2(134, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 140, [134 | __Ss], [__T | __Stack]);
yeccpars2(134, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(135, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {addReq,__1},
 yeccpars2(yeccgoto(ammToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(136, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 679, [136 | __Ss], [__T | __Stack]);
yeccpars2(136, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(137, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 676, [137 | __Ss], [__T | __Stack]);
yeccpars2(137, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(138, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 666, [138 | __Ss], [__T | __Stack]);
yeccpars2(138, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(139, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {emergency,false},
 yeccpars2(yeccgoto(contextProperty, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(140, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {emergency,true},
 yeccpars2(yeccgoto(contextProperty, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(141, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {modReq,__1},
 yeccpars2(yeccgoto(ammToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(142, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {moveReq,__1},
 yeccpars2(yeccgoto(ammToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(143, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 649, [143 | __Ss], [__T | __Stack]);
yeccpars2(143, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(144, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 647, [144 | __Ss], [__T | __Stack]);
yeccpars2(144, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(145, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 597, [145 | __Ss], [__T | __Stack]);
yeccpars2(145, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(146, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 591, [146 | __Ss], [__T | __Stack]);
yeccpars2(146, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(147, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 574, [147 | __Ss], [__T | __Stack]);
yeccpars2(147, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(148, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 569, [148 | __Ss], [__T | __Stack]);
yeccpars2(148, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(570, __Cat, [148 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(149, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(commandRequest, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(150, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 160, [150 | __Ss], [__T | __Stack]);
yeccpars2(150, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(151, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(commandRequest, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(152, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(actionRequestItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(153, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(actionRequestItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(154, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(actionRequestItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(155, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(commandRequest, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(156, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {priority,__1},
 yeccpars2(yeccgoto(contextProperty, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(157, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(commandRequest, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(158, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(commandRequest, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(159, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {topology,__1},
 yeccpars2(yeccgoto(contextProperty, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(160, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [160 | __Ss], [__T | __Stack]);
yeccpars2(160, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(161, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_terminationID(__1),
 yeccpars2(yeccgoto(terminationID, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(162, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 163, [162 | __Ss], [__T | __Stack]);
yeccpars2(162, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(164, __Cat, [162 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(163, 'DigitMapDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 166, [163 | __Ss], [__T | __Stack]);
yeccpars2(163, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 170, [163 | __Ss], [__T | __Stack]);
yeccpars2(163, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 172, [163 | __Ss], [__T | __Stack]);
yeccpars2(163, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 168, [163 | __Ss], [__T | __Stack]);
yeccpars2(163, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 171, [163 | __Ss], [__T | __Stack]);
yeccpars2(163, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 167, [163 | __Ss], [__T | __Stack]);
yeccpars2(163, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 169, [163 | __Ss], [__T | __Stack]);
yeccpars2(163, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 165, [163 | __Ss], [__T | __Stack]);
yeccpars2(163, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(164, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val = begin
   Descs = merge_AmmRequest_descriptors(__4,[]), make_commandRequest(__1,#'AmmRequest'{terminationID = [__3], descriptors = Descs})
  end,
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(ammRequest, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(165, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 464, [165 | __Ss], [__T | __Stack]);
yeccpars2(165, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(166, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_DMD(__1),
 yeccpars2(yeccgoto(digitMapDescriptor, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(167, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 438, [167 | __Ss], [__T | __Stack]);
yeccpars2(167, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(yeccgoto(eventBufferDescriptor, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(168, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 374, [168 | __Ss], [__T | __Stack]);
yeccpars2(168, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'EventsDescriptor'{requestID = asn1_NOVALUE, eventList = []},
 yeccpars2(yeccgoto(eventsDescriptor, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(169, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 304, [169 | __Ss], [__T | __Stack]);
yeccpars2(169, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(170, 'LSBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 284, [170 | __Ss], [__T | __Stack]);
yeccpars2(170, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 283, [170 | __Ss], [__T | __Stack]);
yeccpars2(170, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(171, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 272, [171 | __Ss], [__T | __Stack]);
yeccpars2(171, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(172, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 187, [172 | __Ss], [__T | __Stack]);
yeccpars2(172, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(yeccgoto(signalsDescriptor, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(173, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 182, [173 | __Ss], [__T | __Stack]);
yeccpars2(173, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(183, __Cat, [173 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(174, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {auditDescriptor,__1},
 yeccpars2(yeccgoto(ammParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(175, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {digitMapDescriptor,__1},
 yeccpars2(yeccgoto(ammParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(176, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {eventBufferDescriptor,__1},
 yeccpars2(yeccgoto(ammParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(177, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {eventsDescriptor,__1},
 yeccpars2(yeccgoto(ammParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(178, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {mediaDescriptor,__1},
 yeccpars2(yeccgoto(ammParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(179, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {modemDescriptor,deprecated},
 yeccpars2(yeccgoto(ammParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(180, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {muxDescriptor,__1},
 yeccpars2(yeccgoto(ammParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(181, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {signalsDescriptor,__1},
 yeccpars2(yeccgoto(ammParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(182, 'DigitMapDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 166, [182 | __Ss], [__T | __Stack]);
yeccpars2(182, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 170, [182 | __Ss], [__T | __Stack]);
yeccpars2(182, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 172, [182 | __Ss], [__T | __Stack]);
yeccpars2(182, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 168, [182 | __Ss], [__T | __Stack]);
yeccpars2(182, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 171, [182 | __Ss], [__T | __Stack]);
yeccpars2(182, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 167, [182 | __Ss], [__T | __Stack]);
yeccpars2(182, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 169, [182 | __Ss], [__T | __Stack]);
yeccpars2(182, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 165, [182 | __Ss], [__T | __Stack]);
yeccpars2(182, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(183, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 184, [183 | __Ss], [__T | __Stack]);
yeccpars2(183, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(184, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(ammRequestBody, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(185, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 182, [185 | __Ss], [__T | __Stack]);
yeccpars2(185, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(186, __Cat, [185 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(186, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(ammParameters, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(187, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 188, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [187 | __Ss], [__T | __Stack]);
yeccpars2(187, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(188, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 261, [188 | __Ss], [__T | __Stack]);
yeccpars2(188, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(189, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(signalName, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(190, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_pkgdName(__1),
 yeccpars2(yeccgoto(pkgdName, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(191, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {seqSigList,__1},
 yeccpars2(yeccgoto(signalParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(192, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 200, [192 | __Ss], [__T | __Stack]);
yeccpars2(192, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_signalRequest(__1,[]),
 yeccpars2(yeccgoto(signalRequest, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(193, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 195, [193 | __Ss], [__T | __Stack]);
yeccpars2(193, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(196, __Cat, [193 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(194, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {signal,__1},
 yeccpars2(yeccgoto(signalParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(195, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 188, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [195 | __Ss], [__T | __Stack]);
yeccpars2(195, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(196, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 197, [196 | __Ss], [__T | __Stack]);
yeccpars2(196, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(197, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__3|__4],
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(signalsDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(198, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 195, [198 | __Ss], [__T | __Stack]);
yeccpars2(198, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(199, __Cat, [198 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(199, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(signalParms, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(200, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 205, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 204, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 203, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 202, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 201, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [200 | __Ss], [__T | __Stack]);
yeccpars2(200, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(201, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 259, [201 | __Ss], [__T | __Stack]);
yeccpars2(201, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(202, 'COMMA', __Ss, [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  keepActive,
 yeccpars2(yeccgoto(sigParameter, hd(__Ss)), 'COMMA', __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(202, 'RBRKT', __Ss, [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  keepActive,
 yeccpars2(yeccgoto(sigParameter, hd(__Ss)), 'RBRKT', __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(202, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(203, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 247, [203 | __Ss], [__T | __Stack]);
yeccpars2(203, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(204, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 242, [204 | __Ss], [__T | __Stack]);
yeccpars2(204, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(205, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 239, [205 | __Ss], [__T | __Stack]);
yeccpars2(205, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(206, 'GREATER', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 214, [206 | __Ss], [__T | __Stack]);
yeccpars2(206, 'LESSER', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 215, [206 | __Ss], [__T | __Stack]);
yeccpars2(206, 'NEQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 216, [206 | __Ss], [__T | __Stack]);
yeccpars2(206, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 213, [206 | __Ss], [__T | __Stack]);
yeccpars2(206, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(207, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 208, [207 | __Ss], [__T | __Stack]);
yeccpars2(207, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(209, __Cat, [207 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(208, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 205, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 204, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 203, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 202, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 201, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [208 | __Ss], [__T | __Stack]);
yeccpars2(208, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(209, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 210, [209 | __Ss], [__T | __Stack]);
yeccpars2(209, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(210, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_signalRequest(__1,[__3|__4]),
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(signalRequest, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(211, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 208, [211 | __Ss], [__T | __Stack]);
yeccpars2(211, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(212, __Cat, [211 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(212, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(sigParameters, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(213, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'QuotedChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 218, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'LSBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 224, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 223, [213 | __Ss], [__T | __Stack]);
yeccpars2(213, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(214, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, 'QuotedChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 218, [214 | __Ss], [__T | __Stack]);
yeccpars2(214, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(215, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, 'QuotedChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 218, [215 | __Ss], [__T | __Stack]);
yeccpars2(215, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(216, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, 'QuotedChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 218, [216 | __Ss], [__T | __Stack]);
yeccpars2(216, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(217, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {other,ensure_NAME(__1),__2},
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(sigParameter, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(218, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_value(__1),
 yeccpars2(yeccgoto(value, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(219, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_value(__1),
 yeccpars2(yeccgoto(value, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(220, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'PropertyParm'{value = [__2], extraInfo = {relation,unequalTo}},
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(parmValue, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(221, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'PropertyParm'{value = [__2], extraInfo = {relation,smallerThan}},
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(parmValue, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(222, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'PropertyParm'{value = [__2], extraInfo = {relation,greaterThan}},
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(parmValue, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(223, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, 'QuotedChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 218, [223 | __Ss], [__T | __Stack]);
yeccpars2(223, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(224, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, 'QuotedChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 218, [224 | __Ss], [__T | __Stack]);
yeccpars2(224, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(225, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __2,
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(parmValue, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(226, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'PropertyParm'{value = [__1]},
 yeccpars2(yeccgoto(alternativeValue, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(227, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 229, [227 | __Ss], [__T | __Stack]);
yeccpars2(227, 'COLON', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 228, [227 | __Ss], [__T | __Stack]);
yeccpars2(227, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(230, __Cat, [227 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(228, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, 'QuotedChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 218, [228 | __Ss], [__T | __Stack]);
yeccpars2(228, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(229, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, 'QuotedChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 218, [229 | __Ss], [__T | __Stack]);
yeccpars2(229, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(230, 'RSBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 231, [230 | __Ss], [__T | __Stack]);
yeccpars2(230, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(231, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'PropertyParm'{value = [__2|__3], extraInfo = {sublist,true}},
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(alternativeValue, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(232, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 229, [232 | __Ss], [__T | __Stack]);
yeccpars2(232, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(233, __Cat, [232 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(233, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(valueList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(234, 'RSBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 235, [234 | __Ss], [__T | __Stack]);
yeccpars2(234, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(235, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'PropertyParm'{value = [__2,__4], extraInfo = {range,true}},
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(alternativeValue, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(236, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 229, [236 | __Ss], [__T | __Stack]);
yeccpars2(236, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(237, __Cat, [236 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(237, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 238, [237 | __Ss], [__T | __Stack]);
yeccpars2(237, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(238, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'PropertyParm'{value = [__2|__3], extraInfo = {sublist,false}},
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(alternativeValue, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(239, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [239 | __Ss], [__T | __Stack]);
yeccpars2(239, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(240, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_streamID(__1),
 yeccpars2(yeccgoto(streamID, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(241, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {stream,__3},
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(sigParameter, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(242, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 243, [242 | __Ss], [__T | __Stack]);
yeccpars2(242, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 245, [242 | __Ss], [__T | __Stack]);
yeccpars2(242, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 244, [242 | __Ss], [__T | __Stack]);
yeccpars2(242, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(243, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  brief,
 yeccpars2(yeccgoto(signalType, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(244, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  onOff,
 yeccpars2(yeccgoto(signalType, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(245, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  timeOut,
 yeccpars2(yeccgoto(signalType, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(246, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {signal_type,__3},
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(sigParameter, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(247, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 248, [247 | __Ss], [__T | __Stack]);
yeccpars2(247, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(248, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 251, [248 | __Ss], [__T | __Stack]);
yeccpars2(248, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 250, [248 | __Ss], [__T | __Stack]);
yeccpars2(248, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 249, [248 | __Ss], [__T | __Stack]);
yeccpars2(248, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 252, [248 | __Ss], [__T | __Stack]);
yeccpars2(248, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(249, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  onInterruptByEvent,
 yeccpars2(yeccgoto(notificationReason, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(250, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  onInterruptByNewSignalDescr,
 yeccpars2(yeccgoto(notificationReason, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(251, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  otherReason,
 yeccpars2(yeccgoto(notificationReason, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(252, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  onTimeOut,
 yeccpars2(yeccgoto(notificationReason, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(253, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 254, [253 | __Ss], [__T | __Stack]);
yeccpars2(253, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(255, __Cat, [253 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(254, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 251, [254 | __Ss], [__T | __Stack]);
yeccpars2(254, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 250, [254 | __Ss], [__T | __Stack]);
yeccpars2(254, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 249, [254 | __Ss], [__T | __Stack]);
yeccpars2(254, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 252, [254 | __Ss], [__T | __Stack]);
yeccpars2(254, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(255, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 256, [255 | __Ss], [__T | __Stack]);
yeccpars2(255, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(256, __Cat, __Ss,  [__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {notify_completion,[__4|__5]},
 __Nss = lists:nthtail(5, __Ss),
 yeccpars2(yeccgoto(sigParameter, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(257, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 254, [257 | __Ss], [__T | __Stack]);
yeccpars2(257, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(258, __Cat, [257 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(258, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(notificationReasons, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(259, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [259 | __Ss], [__T | __Stack]);
yeccpars2(259, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(260, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {duration,ensure_uint16(__3)},
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(sigParameter, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(261, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [261 | __Ss], [__T | __Stack]);
yeccpars2(261, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(262, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_uint16(__1),
 yeccpars2(yeccgoto(signalListId, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(263, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 264, [263 | __Ss], [__T | __Stack]);
yeccpars2(263, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(264, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [264 | __Ss], [__T | __Stack]);
yeccpars2(264, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(265, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 267, [265 | __Ss], [__T | __Stack]);
yeccpars2(265, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(268, __Cat, [265 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(266, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(signalListParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(267, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [267 | __Ss], [__T | __Stack]);
yeccpars2(267, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(268, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 269, [268 | __Ss], [__T | __Stack]);
yeccpars2(268, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(269, __Cat, __Ss,  [__7,__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'SeqSigList'{id = ensure_uint16(__3), signalList = [__5|__6]},
 __Nss = lists:nthtail(6, __Ss),
 yeccpars2(yeccgoto(signalList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(270, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 267, [270 | __Ss], [__T | __Stack]);
yeccpars2(270, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(271, __Cat, [270 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(271, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(signalListParms, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(272, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [272 | __Ss], [__T | __Stack]);
yeccpars2(272, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(273, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 275, [273 | __Ss], [__T | __Stack]);
yeccpars2(273, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(274, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_muxType(__1),
 yeccpars2(yeccgoto(muxType, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(275, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [275 | __Ss], [__T | __Stack]);
yeccpars2(275, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(276, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'MuxDescriptor'{muxType = __3, termList = __4},
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(muxDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(277, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 278, [277 | __Ss], [__T | __Stack]);
yeccpars2(277, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(279, __Cat, [277 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(278, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [278 | __Ss], [__T | __Stack]);
yeccpars2(278, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(279, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 280, [279 | __Ss], [__T | __Stack]);
yeccpars2(279, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(280, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(terminationIDList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(281, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 278, [281 | __Ss], [__T | __Stack]);
yeccpars2(281, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(282, __Cat, [281 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(282, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(terminationIDListRepeat, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(283, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [283 | __Ss], [__T | __Stack]);
yeccpars2(283, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(284, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [284 | __Ss], [__T | __Stack]);
yeccpars2(284, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(285, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 287, [285 | __Ss], [__T | __Stack]);
yeccpars2(285, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  '$undefined',
 yeccpars2(288, __Cat, [285 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(286, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  '$undefined',
 yeccpars2(yeccgoto(modemType, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(287, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [287 | __Ss], [__T | __Stack]);
yeccpars2(287, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(288, 'RSBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 289, [288 | __Ss], [__T | __Stack]);
yeccpars2(288, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(289, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 290, [289 | __Ss], [__T | __Stack]);
yeccpars2(289, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(291, __Cat, [289 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(290, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [290 | __Ss], [__T | __Stack]);
yeccpars2(290, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(291, __Cat, __Ss,  [__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  '$undefined',
 __Nss = lists:nthtail(5, __Ss),
 yeccpars2(yeccgoto(modemDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(292, 'GREATER', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 214, [292 | __Ss], [__T | __Stack]);
yeccpars2(292, 'LESSER', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 215, [292 | __Ss], [__T | __Stack]);
yeccpars2(292, 'NEQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 216, [292 | __Ss], [__T | __Stack]);
yeccpars2(292, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 213, [292 | __Ss], [__T | __Stack]);
yeccpars2(292, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(293, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 294, [293 | __Ss], [__T | __Stack]);
yeccpars2(293, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(295, __Cat, [293 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(294, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [294 | __Ss], [__T | __Stack]);
yeccpars2(294, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(295, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 296, [295 | __Ss], [__T | __Stack]);
yeccpars2(295, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(296, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(optPropertyParms, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(297, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 294, [297 | __Ss], [__T | __Stack]);
yeccpars2(297, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(298, __Cat, [297 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(298, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(propertyParms, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(299, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  setelement(#'PropertyParm'.name,__2,__1),
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(propertyParm, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(300, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 287, [300 | __Ss], [__T | __Stack]);
yeccpars2(300, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  '$undefined',
 yeccpars2(301, __Cat, [300 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(301, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  '$undefined',
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(modemTypeList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(302, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 290, [302 | __Ss], [__T | __Stack]);
yeccpars2(302, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(303, __Cat, [302 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(303, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  '$undefined',
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(modemDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(304, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 309, [304 | __Ss], [__T | __Stack]);
yeccpars2(304, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 305, [304 | __Ss], [__T | __Stack]);
yeccpars2(304, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 308, [304 | __Ss], [__T | __Stack]);
yeccpars2(304, 'RemoteDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 307, [304 | __Ss], [__T | __Stack]);
yeccpars2(304, 'LocalDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 306, [304 | __Ss], [__T | __Stack]);
yeccpars2(304, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(305, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 350, [305 | __Ss], [__T | __Stack]);
yeccpars2(305, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(306, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {local,#'LocalRemoteDescriptor'{propGrps = ensure_prop_groups(__1)}},
 yeccpars2(yeccgoto(streamParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(307, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {remote,#'LocalRemoteDescriptor'{propGrps = ensure_prop_groups(__1)}},
 yeccpars2(yeccgoto(streamParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(308, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 341, [308 | __Ss], [__T | __Stack]);
yeccpars2(308, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(309, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 320, [309 | __Ss], [__T | __Stack]);
yeccpars2(309, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(310, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {control,__1},
 yeccpars2(yeccgoto(streamParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(311, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 315, [311 | __Ss], [__T | __Stack]);
yeccpars2(311, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(316, __Cat, [311 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(312, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {streamDescriptor,__1},
 yeccpars2(yeccgoto(mediaParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(313, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {streamParm,__1},
 yeccpars2(yeccgoto(mediaParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(314, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {termState,__1},
 yeccpars2(yeccgoto(mediaParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(315, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 309, [315 | __Ss], [__T | __Stack]);
yeccpars2(315, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 305, [315 | __Ss], [__T | __Stack]);
yeccpars2(315, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 308, [315 | __Ss], [__T | __Stack]);
yeccpars2(315, 'RemoteDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 307, [315 | __Ss], [__T | __Stack]);
yeccpars2(315, 'LocalDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 306, [315 | __Ss], [__T | __Stack]);
yeccpars2(315, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(316, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 317, [316 | __Ss], [__T | __Stack]);
yeccpars2(316, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(317, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_mediaDescriptor([__3|__4]),
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(mediaDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(318, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 315, [318 | __Ss], [__T | __Stack]);
yeccpars2(318, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(319, __Cat, [318 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(319, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(mediaParmList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(320, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 322, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 321, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [320 | __Ss], [__T | __Stack]);
yeccpars2(320, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(321, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 337, [321 | __Ss], [__T | __Stack]);
yeccpars2(321, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(322, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 332, [322 | __Ss], [__T | __Stack]);
yeccpars2(322, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(323, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {eventBufferControl,__1},
 yeccpars2(yeccgoto(terminationStateParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(324, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {propertyParm,__1},
 yeccpars2(yeccgoto(terminationStateParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(325, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {serviceState,__1},
 yeccpars2(yeccgoto(terminationStateParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(326, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 327, [326 | __Ss], [__T | __Stack]);
yeccpars2(326, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(328, __Cat, [326 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(327, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 322, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 321, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [327 | __Ss], [__T | __Stack]);
yeccpars2(327, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(328, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 329, [328 | __Ss], [__T | __Stack]);
yeccpars2(328, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(329, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_terminationStateDescriptor([__3|__4]),
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(terminationStateDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(330, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 327, [330 | __Ss], [__T | __Stack]);
yeccpars2(330, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(331, __Cat, [330 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(331, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(terminationStateParms, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(332, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 333, [332 | __Ss], [__T | __Stack]);
yeccpars2(332, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 334, [332 | __Ss], [__T | __Stack]);
yeccpars2(332, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 335, [332 | __Ss], [__T | __Stack]);
yeccpars2(332, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(333, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  inSvc,
 yeccpars2(yeccgoto(serviceState, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(334, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  outOfSvc,
 yeccpars2(yeccgoto(serviceState, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(335, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  test,
 yeccpars2(yeccgoto(serviceState, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(336, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __3,
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(serviceStates, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(337, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 338, [337 | __Ss], [__T | __Stack]);
yeccpars2(337, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 339, [337 | __Ss], [__T | __Stack]);
yeccpars2(337, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(338, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  lockStep,
 yeccpars2(yeccgoto(eventBufferControlState, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(339, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  off,
 yeccpars2(yeccgoto(eventBufferControlState, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(340, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __3,
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(eventBufferControl, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(341, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [341 | __Ss], [__T | __Stack]);
yeccpars2(341, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(342, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 343, [342 | __Ss], [__T | __Stack]);
yeccpars2(342, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(343, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 305, [343 | __Ss], [__T | __Stack]);
yeccpars2(343, 'RemoteDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 307, [343 | __Ss], [__T | __Stack]);
yeccpars2(343, 'LocalDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 306, [343 | __Ss], [__T | __Stack]);
yeccpars2(343, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(344, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 345, [344 | __Ss], [__T | __Stack]);
yeccpars2(344, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(346, __Cat, [344 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(345, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 305, [345 | __Ss], [__T | __Stack]);
yeccpars2(345, 'RemoteDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 307, [345 | __Ss], [__T | __Stack]);
yeccpars2(345, 'LocalDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 306, [345 | __Ss], [__T | __Stack]);
yeccpars2(345, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(346, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 347, [346 | __Ss], [__T | __Stack]);
yeccpars2(346, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(347, __Cat, __Ss,  [__7,__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'StreamDescriptor'{streamID = __3, streamParms = merge_streamParms([__5|__6])},
 __Nss = lists:nthtail(6, __Ss),
 yeccpars2(yeccgoto(streamDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(348, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 345, [348 | __Ss], [__T | __Stack]);
yeccpars2(348, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(349, __Cat, [348 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(349, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(streamParmList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(350, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 353, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 352, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 351, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [350 | __Ss], [__T | __Stack]);
yeccpars2(350, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(351, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 367, [351 | __Ss], [__T | __Stack]);
yeccpars2(351, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(352, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 365, [352 | __Ss], [__T | __Stack]);
yeccpars2(352, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(353, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 361, [353 | __Ss], [__T | __Stack]);
yeccpars2(353, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(354, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 356, [354 | __Ss], [__T | __Stack]);
yeccpars2(354, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(357, __Cat, [354 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(355, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {prop,__1},
 yeccpars2(yeccgoto(localParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(356, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 353, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 352, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 351, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [356 | __Ss], [__T | __Stack]);
yeccpars2(356, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(357, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 358, [357 | __Ss], [__T | __Stack]);
yeccpars2(357, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(358, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__3|__4],
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(localControlDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(359, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 356, [359 | __Ss], [__T | __Stack]);
yeccpars2(359, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(360, __Cat, [359 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(360, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(localParmList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(361, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 362, [361 | __Ss], [__T | __Stack]);
yeccpars2(361, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 363, [361 | __Ss], [__T | __Stack]);
yeccpars2(361, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(362, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  false,
 yeccpars2(yeccgoto(onOrOff, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(363, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  true,
 yeccpars2(yeccgoto(onOrOff, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(364, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {value,__3},
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(localParm, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(365, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 362, [365 | __Ss], [__T | __Stack]);
yeccpars2(365, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 363, [365 | __Ss], [__T | __Stack]);
yeccpars2(365, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(366, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {group,__3},
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(localParm, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(367, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 369, [367 | __Ss], [__T | __Stack]);
yeccpars2(367, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 368, [367 | __Ss], [__T | __Stack]);
yeccpars2(367, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 372, [367 | __Ss], [__T | __Stack]);
yeccpars2(367, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 370, [367 | __Ss], [__T | __Stack]);
yeccpars2(367, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 371, [367 | __Ss], [__T | __Stack]);
yeccpars2(367, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(368, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  inactive,
 yeccpars2(yeccgoto(streamModes, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(369, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  loopBack,
 yeccpars2(yeccgoto(streamModes, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(370, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  recvOnly,
 yeccpars2(yeccgoto(streamModes, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(371, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  sendOnly,
 yeccpars2(yeccgoto(streamModes, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(372, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  sendRecv,
 yeccpars2(yeccgoto(streamModes, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(373, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {mode,__3},
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(localParm, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(374, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [374 | __Ss], [__T | __Stack]);
yeccpars2(374, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(375, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 377, [375 | __Ss], [__T | __Stack]);
yeccpars2(375, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(376, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_requestID(__1),
 yeccpars2(yeccgoto(requestID, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(377, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [377 | __Ss], [__T | __Stack]);
yeccpars2(377, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(378, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 385, [378 | __Ss], [__T | __Stack]);
yeccpars2(378, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  #'RequestedEvent'{evParList = []},
 yeccpars2(386, __Cat, [378 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(379, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 380, [379 | __Ss], [__T | __Stack]);
yeccpars2(379, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(381, __Cat, [379 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(380, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [380 | __Ss], [__T | __Stack]);
yeccpars2(380, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(381, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 382, [381 | __Ss], [__T | __Stack]);
yeccpars2(381, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(382, __Cat, __Ss,  [__7,__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'EventsDescriptor'{requestID = __3, eventList = [__5|__6]},
 __Nss = lists:nthtail(6, __Ss),
 yeccpars2(yeccgoto(eventsDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(383, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 380, [383 | __Ss], [__T | __Stack]);
yeccpars2(383, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(384, __Cat, [383 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(384, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(requestedEvents, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(385, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 389, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 388, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, 'DigitMapDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 387, [385 | __Ss], [__T | __Stack]);
yeccpars2(385, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(386, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  setelement(#'RequestedEvent'.pkgdName,__2,__1),
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(requestedEvent, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(387, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_eventDM(__1),
 yeccpars2(yeccgoto(eventDM, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(388, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 403, [388 | __Ss], [__T | __Stack]);
yeccpars2(388, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(389, 'COMMA', __Ss, [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  keepActive,
 yeccpars2(yeccgoto(eventParameter, hd(__Ss)), 'COMMA', __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(389, 'RBRKT', __Ss, [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  keepActive,
 yeccpars2(yeccgoto(eventParameter, hd(__Ss)), 'RBRKT', __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(389, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(390, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(eventParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(391, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(eventParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(392, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(eventParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(393, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 398, [393 | __Ss], [__T | __Stack]);
yeccpars2(393, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(399, __Cat, [393 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(394, 'GREATER', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 214, [394 | __Ss], [__T | __Stack]);
yeccpars2(394, 'LESSER', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 215, [394 | __Ss], [__T | __Stack]);
yeccpars2(394, 'NEQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 216, [394 | __Ss], [__T | __Stack]);
yeccpars2(394, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 213, [394 | __Ss], [__T | __Stack]);
yeccpars2(394, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(395, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(eventParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(396, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_NAME(__1),
 yeccpars2(yeccgoto(eventParameterName, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(397, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  select_stream_or_other(__1,__2),
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(eventStreamOrOther, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(398, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 389, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 388, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, 'DigitMapDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 387, [398 | __Ss], [__T | __Stack]);
yeccpars2(398, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(399, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 400, [399 | __Ss], [__T | __Stack]);
yeccpars2(399, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(400, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_eventParameters([__2|__3]),
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(requestedEventBody, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(401, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 398, [401 | __Ss], [__T | __Stack]);
yeccpars2(401, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(402, __Cat, [401 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(402, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(eventParameters, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(403, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 172, [403 | __Ss], [__T | __Stack]);
yeccpars2(403, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 404, [403 | __Ss], [__T | __Stack]);
yeccpars2(403, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(404, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 412, [404 | __Ss], [__T | __Stack]);
yeccpars2(404, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'SecondEventsDescriptor'{requestID = asn1_NOVALUE, eventList = []},
 yeccpars2(yeccgoto(embedFirst, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(405, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 411, [405 | __Ss], [__T | __Stack]);
yeccpars2(405, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(406, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 408, [406 | __Ss], [__T | __Stack]);
yeccpars2(406, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 407, [406 | __Ss], [__T | __Stack]);
yeccpars2(406, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(407, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 404, [407 | __Ss], [__T | __Stack]);
yeccpars2(407, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(408, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {embed,__3,asn1_NOVALUE},
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(embedWithSig, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(409, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 410, [409 | __Ss], [__T | __Stack]);
yeccpars2(409, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(410, __Cat, __Ss,  [__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {embed,__3,__5},
 __Nss = lists:nthtail(5, __Ss),
 yeccpars2(yeccgoto(embedWithSig, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(411, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {embed,asn1_NOVALUE,__3},
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(embedNoSig, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(412, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [412 | __Ss], [__T | __Stack]);
yeccpars2(412, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(413, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 414, [413 | __Ss], [__T | __Stack]);
yeccpars2(413, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(414, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [414 | __Ss], [__T | __Stack]);
yeccpars2(414, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(415, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 422, [415 | __Ss], [__T | __Stack]);
yeccpars2(415, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  #'SecondRequestedEvent'{evParList = []},
 yeccpars2(423, __Cat, [415 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(416, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 417, [416 | __Ss], [__T | __Stack]);
yeccpars2(416, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(418, __Cat, [416 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(417, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [417 | __Ss], [__T | __Stack]);
yeccpars2(417, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(418, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 419, [418 | __Ss], [__T | __Stack]);
yeccpars2(418, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(419, __Cat, __Ss,  [__7,__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'SecondEventsDescriptor'{requestID = __3, eventList = [__5|__6]},
 __Nss = lists:nthtail(6, __Ss),
 yeccpars2(yeccgoto(embedFirst, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(420, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 417, [420 | __Ss], [__T | __Stack]);
yeccpars2(420, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(421, __Cat, [420 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(421, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(secondRequestedEvents, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(422, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 425, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 424, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, 'DigitMapDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 387, [422 | __Ss], [__T | __Stack]);
yeccpars2(422, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(423, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  setelement(#'SecondRequestedEvent'.pkgdName,__2,__1),
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(secondRequestedEvent, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(424, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 435, [424 | __Ss], [__T | __Stack]);
yeccpars2(424, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(425, 'COMMA', __Ss, [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  keepActive,
 yeccpars2(yeccgoto(secondEventParameter, hd(__Ss)), 'COMMA', __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(425, 'RBRKT', __Ss, [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  keepActive,
 yeccpars2(yeccgoto(secondEventParameter, hd(__Ss)), 'RBRKT', __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(425, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(426, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(secondEventParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(427, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(secondEventParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(428, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(secondEventParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(429, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 430, [429 | __Ss], [__T | __Stack]);
yeccpars2(429, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(431, __Cat, [429 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(430, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 425, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 424, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, 'DigitMapDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 387, [430 | __Ss], [__T | __Stack]);
yeccpars2(430, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(431, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 432, [431 | __Ss], [__T | __Stack]);
yeccpars2(431, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(432, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_secondEventParameters([__2|__3]),
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(secondRequestedEventBody, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(433, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 430, [433 | __Ss], [__T | __Stack]);
yeccpars2(433, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(434, __Cat, [433 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(434, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(secondEventParameters, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(435, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 172, [435 | __Ss], [__T | __Stack]);
yeccpars2(435, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(436, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 437, [436 | __Ss], [__T | __Stack]);
yeccpars2(436, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(437, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {second_embed,__3},
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(embedSig, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(438, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 439, [438 | __Ss], [__T | __Stack]);
yeccpars2(438, 'SEP', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 1, [438 | __Ss], [__T | __Stack]);
yeccpars2(438, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  no_sep,
 yeccpars2(442, __Cat, [438 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(439, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_timeStamp(__1),
 yeccpars2(yeccgoto(timeStamp, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(440, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 459, [440 | __Ss], [__T | __Stack]);
yeccpars2(440, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(460, __Cat, [440 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(441, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_eventSpec(__1),
 yeccpars2(yeccgoto(eventSpec, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(442, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [442 | __Ss], [__T | __Stack]);
yeccpars2(442, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(443, 'SEP', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 1, [443 | __Ss], [__T | __Stack]);
yeccpars2(443, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  no_sep,
 yeccpars2(444, __Cat, [443 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(444, 'COLON', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 445, [444 | __Ss], [__T | __Stack]);
yeccpars2(444, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(445, 'SEP', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 1, [445 | __Ss], [__T | __Stack]);
yeccpars2(445, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  no_sep,
 yeccpars2(446, __Cat, [445 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(446, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [446 | __Ss], [__T | __Stack]);
yeccpars2(446, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(447, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 448, [447 | __Ss], [__T | __Stack]);
yeccpars2(447, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(449, __Cat, [447 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(448, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [448 | __Ss], [__T | __Stack]);
yeccpars2(448, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(449, __Cat, __Ss,  [__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_observed_event(__6,__5,__1),
 __Nss = lists:nthtail(5, __Ss),
 yeccpars2(yeccgoto(observedEvent, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(450, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(observedEventParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(451, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 452, [451 | __Ss], [__T | __Stack]);
yeccpars2(451, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(453, __Cat, [451 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(452, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [452 | __Ss], [__T | __Stack]);
yeccpars2(452, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(453, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 454, [453 | __Ss], [__T | __Stack]);
yeccpars2(453, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(454, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(observedEventBody, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(455, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 452, [455 | __Ss], [__T | __Stack]);
yeccpars2(455, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(456, __Cat, [455 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(456, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(observedEventParameters, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(457, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 448, [457 | __Ss], [__T | __Stack]);
yeccpars2(457, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(458, __Cat, [457 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(458, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_observed_event(__3,__2,asn1_NOVALUE),
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(observedEvent, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(459, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 439, [459 | __Ss], [__T | __Stack]);
yeccpars2(459, 'SEP', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 1, [459 | __Ss], [__T | __Stack]);
yeccpars2(459, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  no_sep,
 yeccpars2(442, __Cat, [459 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(460, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 461, [460 | __Ss], [__T | __Stack]);
yeccpars2(460, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(461, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__3|__4],
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(eventBufferDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(462, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 459, [462 | __Ss], [__T | __Stack]);
yeccpars2(462, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(463, __Cat, [462 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(463, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(eventSpecList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(464, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 473, [464 | __Ss], [__T | __Stack]);
yeccpars2(464, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 475, [464 | __Ss], [__T | __Stack]);
yeccpars2(464, 'DigitMapDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 465, [464 | __Ss], [__T | __Stack]);
yeccpars2(464, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 474, [464 | __Ss], [__T | __Stack]);
yeccpars2(464, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 468, [464 | __Ss], [__T | __Stack]);
yeccpars2(464, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 467, [464 | __Ss], [__T | __Stack]);
yeccpars2(464, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 469, [464 | __Ss], [__T | __Stack]);
yeccpars2(464, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 472, [464 | __Ss], [__T | __Stack]);
yeccpars2(464, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 466, [464 | __Ss], [__T | __Stack]);
yeccpars2(464, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 470, [464 | __Ss], [__T | __Stack]);
yeccpars2(464, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 471, [464 | __Ss], [__T | __Stack]);
yeccpars2(464, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  asn1_NOVALUE,
 yeccpars2(476, __Cat, [464 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(465, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_IADMD(__1),
 yeccpars2(yeccgoto(indAuddigitMapDescriptor, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(466, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  digitMapToken,
 yeccpars2(yeccgoto(auditReturnItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(467, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 551, [467 | __Ss], [__T | __Stack]);
yeccpars2(467, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  eventBufferToken,
 yeccpars2(yeccgoto(auditItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(468, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 545, [468 | __Ss], [__T | __Stack]);
yeccpars2(468, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  eventsToken,
 yeccpars2(yeccgoto(auditItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(469, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 518, [469 | __Ss], [__T | __Stack]);
yeccpars2(469, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  mediaToken,
 yeccpars2(yeccgoto(auditReturnItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(470, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  modemToken,
 yeccpars2(yeccgoto(auditReturnItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(471, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  muxToken,
 yeccpars2(yeccgoto(auditReturnItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(472, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  observedEventsToken,
 yeccpars2(yeccgoto(auditReturnItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(473, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 514, [473 | __Ss], [__T | __Stack]);
yeccpars2(473, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  packagesToken,
 yeccpars2(yeccgoto(auditReturnItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(474, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 501, [474 | __Ss], [__T | __Stack]);
yeccpars2(474, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  signalsToken,
 yeccpars2(yeccgoto(auditItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(475, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 498, [475 | __Ss], [__T | __Stack]);
yeccpars2(475, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  statsToken,
 yeccpars2(yeccgoto(auditReturnItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(476, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 568, [476 | __Ss], [__T | __Stack]);
yeccpars2(476, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(477, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 564, [477 | __Ss], [__T | __Stack]);
yeccpars2(477, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(565, __Cat, [477 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(478, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(auditItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(479, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 488, [479 | __Ss], [__T | __Stack]);
yeccpars2(479, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(489, __Cat, [479 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(480, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {indAudDigitMapDescriptor,__1},
 yeccpars2(yeccgoto(indAudauditReturnParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(481, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {indAudEventBufferDescriptor,__1},
 yeccpars2(yeccgoto(indAudauditReturnParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(482, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {indAudEventsDescriptor,__1},
 yeccpars2(yeccgoto(indAudauditReturnParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(483, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {indAudMediaDescriptor,__1},
 yeccpars2(yeccgoto(indAudauditReturnParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(484, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {indAudPackagesDescriptor,__1},
 yeccpars2(yeccgoto(indAudauditReturnParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(485, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {indAudSignalsDescriptor,__1},
 yeccpars2(yeccgoto(indAudauditReturnParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(486, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {indAudStatisticsDescriptor,__1},
 yeccpars2(yeccgoto(indAudauditReturnParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(487, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {terminationAudit,__1},
 yeccpars2(yeccgoto(auditItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(488, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 493, [488 | __Ss], [__T | __Stack]);
yeccpars2(488, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 495, [488 | __Ss], [__T | __Stack]);
yeccpars2(488, 'DigitMapDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 465, [488 | __Ss], [__T | __Stack]);
yeccpars2(488, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 494, [488 | __Ss], [__T | __Stack]);
yeccpars2(488, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 491, [488 | __Ss], [__T | __Stack]);
yeccpars2(488, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 490, [488 | __Ss], [__T | __Stack]);
yeccpars2(488, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 492, [488 | __Ss], [__T | __Stack]);
yeccpars2(488, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(489, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__1|__2],
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(indAudterminationAudit, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(490, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 551, [490 | __Ss], [__T | __Stack]);
yeccpars2(490, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(491, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 545, [491 | __Ss], [__T | __Stack]);
yeccpars2(491, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(492, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 518, [492 | __Ss], [__T | __Stack]);
yeccpars2(492, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(493, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 514, [493 | __Ss], [__T | __Stack]);
yeccpars2(493, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(494, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 501, [494 | __Ss], [__T | __Stack]);
yeccpars2(494, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(495, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 498, [495 | __Ss], [__T | __Stack]);
yeccpars2(495, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(496, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 488, [496 | __Ss], [__T | __Stack]);
yeccpars2(496, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(497, __Cat, [496 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(497, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(indAudterminationAuditList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(498, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [498 | __Ss], [__T | __Stack]);
yeccpars2(498, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(499, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 500, [499 | __Ss], [__T | __Stack]);
yeccpars2(499, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(500, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'IndAudStatisticsDescriptor'{statName = __3},
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(indAudstatisticsDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(501, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 504, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 503, [501 | __Ss], [__T | __Stack]);
yeccpars2(501, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(502, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __2,
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(indAudsignalsDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(503, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  asn1_NOVALUE,
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(optIndAudsignalParm, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(504, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 509, [504 | __Ss], [__T | __Stack]);
yeccpars2(504, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(505, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {seqSigList,__1},
 yeccpars2(yeccgoto(indAudsignalParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(506, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 508, [506 | __Ss], [__T | __Stack]);
yeccpars2(506, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(507, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {signal,ensure_indAudSignal(__1)},
 yeccpars2(yeccgoto(indAudsignalParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(508, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __2,
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(optIndAudsignalParm, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(509, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [509 | __Ss], [__T | __Stack]);
yeccpars2(509, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(510, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 511, [510 | __Ss], [__T | __Stack]);
yeccpars2(510, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(511, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [511 | __Ss], [__T | __Stack]);
yeccpars2(511, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(512, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 513, [512 | __Ss], [__T | __Stack]);
yeccpars2(512, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(513, __Cat, __Ss,  [__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'IndAudSeqSigList'{id = ensure_uint16(__3), signalList = ensure_indAudSignalListParm(__5)},
 __Nss = lists:nthtail(5, __Ss),
 yeccpars2(yeccgoto(indAudsignalList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(514, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [514 | __Ss], [__T | __Stack]);
yeccpars2(514, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(515, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 517, [515 | __Ss], [__T | __Stack]);
yeccpars2(515, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(516, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_packagesItem(__1),
 yeccpars2(yeccgoto(packagesItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(517, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_indAudPackagesDescriptor(__3),
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(indAudpackagesDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(518, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 521, [518 | __Ss], [__T | __Stack]);
yeccpars2(518, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 519, [518 | __Ss], [__T | __Stack]);
yeccpars2(518, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 520, [518 | __Ss], [__T | __Stack]);
yeccpars2(518, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(519, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 537, [519 | __Ss], [__T | __Stack]);
yeccpars2(519, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(520, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 532, [520 | __Ss], [__T | __Stack]);
yeccpars2(520, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(521, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 528, [521 | __Ss], [__T | __Stack]);
yeccpars2(521, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(522, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'IndAudStreamParms'{localControlDescriptor = __1},
 yeccpars2(yeccgoto(indAudstreamParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(523, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 527, [523 | __Ss], [__T | __Stack]);
yeccpars2(523, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(524, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {streamDescr,__1},
 yeccpars2(yeccgoto(indAudmediaParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(525, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {streamParm,__1},
 yeccpars2(yeccgoto(indAudmediaParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(526, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {termStateDescr,__1},
 yeccpars2(yeccgoto(indAudmediaParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(527, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_indAudMediaDescriptor(__3),
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(indAudmediaDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(528, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [528 | __Ss], [__T | __Stack]);
yeccpars2(528, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(529, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 531, [529 | __Ss], [__T | __Stack]);
yeccpars2(529, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(530, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_indAudTerminationStateParm(__1),
 yeccpars2(yeccgoto(indAudterminationStateParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(531, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_indAudTerminationStateDescriptor(__3),
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(indAudterminationStateDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(532, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [532 | __Ss], [__T | __Stack]);
yeccpars2(532, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(533, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 534, [533 | __Ss], [__T | __Stack]);
yeccpars2(533, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(534, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 519, [534 | __Ss], [__T | __Stack]);
yeccpars2(534, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(535, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 536, [535 | __Ss], [__T | __Stack]);
yeccpars2(535, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(536, __Cat, __Ss,  [__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'IndAudStreamDescriptor'{streamID = __3, streamParms = __5},
 __Nss = lists:nthtail(5, __Ss),
 yeccpars2(yeccgoto(indAudstreamDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(537, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [537 | __Ss], [__T | __Stack]);
yeccpars2(537, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(538, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 540, [538 | __Ss], [__T | __Stack]);
yeccpars2(538, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(541, __Cat, [538 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(539, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_indAudLocalParm(__1),
 yeccpars2(yeccgoto(indAudlocalParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(540, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [540 | __Ss], [__T | __Stack]);
yeccpars2(540, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(541, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 542, [541 | __Ss], [__T | __Stack]);
yeccpars2(541, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(542, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_indAudLocalControlDescriptor([__3|__4]),
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(indAudlocalControlDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(543, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 540, [543 | __Ss], [__T | __Stack]);
yeccpars2(543, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(544, __Cat, [543 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(544, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(indAudlocalParmList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(545, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [545 | __Ss], [__T | __Stack]);
yeccpars2(545, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(546, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 547, [546 | __Ss], [__T | __Stack]);
yeccpars2(546, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(547, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [547 | __Ss], [__T | __Stack]);
yeccpars2(547, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(548, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 550, [548 | __Ss], [__T | __Stack]);
yeccpars2(548, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(549, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(indAudrequestedEvent, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(550, __Cat, __Ss,  [__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'IndAudEventsDescriptor'{requestID = __3, pkgdName = __5},
 __Nss = lists:nthtail(5, __Ss),
 yeccpars2(yeccgoto(indAudeventsDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(551, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [551 | __Ss], [__T | __Stack]);
yeccpars2(551, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(552, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 563, [552 | __Ss], [__T | __Stack]);
yeccpars2(552, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(553, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 554, [553 | __Ss], [__T | __Stack]);
yeccpars2(553, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  asn1_NOVALUE,
 yeccpars2(555, __Cat, [553 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(554, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 556, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [554 | __Ss], [__T | __Stack]);
yeccpars2(554, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(555, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_indAudEventBufferDescriptor(__1,__2),
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(indAudeventSpec, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(556, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 561, [556 | __Ss], [__T | __Stack]);
yeccpars2(556, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(557, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {eventParameterName,__1},
 yeccpars2(yeccgoto(indAudeventSpecParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(558, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {streamID,__1},
 yeccpars2(yeccgoto(indAudeventSpecParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(559, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 560, [559 | __Ss], [__T | __Stack]);
yeccpars2(559, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(560, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __2,
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(optIndAudeventSpecParameter, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(561, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [561 | __Ss], [__T | __Stack]);
yeccpars2(561, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(562, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __3,
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(eventStream, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(563, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __3,
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(indAudeventBufferDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(564, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 473, [564 | __Ss], [__T | __Stack]);
yeccpars2(564, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 475, [564 | __Ss], [__T | __Stack]);
yeccpars2(564, 'DigitMapDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 465, [564 | __Ss], [__T | __Stack]);
yeccpars2(564, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 474, [564 | __Ss], [__T | __Stack]);
yeccpars2(564, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 468, [564 | __Ss], [__T | __Stack]);
yeccpars2(564, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 467, [564 | __Ss], [__T | __Stack]);
yeccpars2(564, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 469, [564 | __Ss], [__T | __Stack]);
yeccpars2(564, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 472, [564 | __Ss], [__T | __Stack]);
yeccpars2(564, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 466, [564 | __Ss], [__T | __Stack]);
yeccpars2(564, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 470, [564 | __Ss], [__T | __Stack]);
yeccpars2(564, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 471, [564 | __Ss], [__T | __Stack]);
yeccpars2(564, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(565, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__1|__2],
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(auditDescriptorBody, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(566, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 564, [566 | __Ss], [__T | __Stack]);
yeccpars2(566, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(567, __Cat, [566 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(567, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(auditItemList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(568, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_auditDescriptor(__3),
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(auditDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(569, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 144, [569 | __Ss], [__T | __Stack]);
yeccpars2(569, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 147, [569 | __Ss], [__T | __Stack]);
yeccpars2(569, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 145, [569 | __Ss], [__T | __Stack]);
yeccpars2(569, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 143, [569 | __Ss], [__T | __Stack]);
yeccpars2(569, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 136, [569 | __Ss], [__T | __Stack]);
yeccpars2(569, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 137, [569 | __Ss], [__T | __Stack]);
yeccpars2(569, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 146, [569 | __Ss], [__T | __Stack]);
yeccpars2(569, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 141, [569 | __Ss], [__T | __Stack]);
yeccpars2(569, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 142, [569 | __Ss], [__T | __Stack]);
yeccpars2(569, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 135, [569 | __Ss], [__T | __Stack]);
yeccpars2(569, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 138, [569 | __Ss], [__T | __Stack]);
yeccpars2(569, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 139, [569 | __Ss], [__T | __Stack]);
yeccpars2(569, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 140, [569 | __Ss], [__T | __Stack]);
yeccpars2(569, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(570, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 571, [570 | __Ss], [__T | __Stack]);
yeccpars2(570, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(571, __Cat, __Ss,  [__7,__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_action_requests(__3,[__5|__6]),
 __Nss = lists:nthtail(6, __Ss),
 yeccpars2(yeccgoto(actionRequest, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(572, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 569, [572 | __Ss], [__T | __Stack]);
yeccpars2(572, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(573, __Cat, [572 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(573, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(actionRequestItems, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(574, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [574 | __Ss], [__T | __Stack]);
yeccpars2(574, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(575, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 583, [575 | __Ss], [__T | __Stack]);
yeccpars2(575, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(576, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(terminationA, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(577, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 578, [577 | __Ss], [__T | __Stack]);
yeccpars2(577, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(579, __Cat, [577 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(578, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [578 | __Ss], [__T | __Stack]);
yeccpars2(578, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(579, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 580, [579 | __Ss], [__T | __Stack]);
yeccpars2(579, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(580, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__3|__4],
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(topologyDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(581, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 578, [581 | __Ss], [__T | __Stack]);
yeccpars2(581, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(582, __Cat, [581 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(582, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(topologyTripleList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(583, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [583 | __Ss], [__T | __Stack]);
yeccpars2(583, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(584, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 586, [584 | __Ss], [__T | __Stack]);
yeccpars2(584, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(585, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __1,
 yeccpars2(yeccgoto(terminationB, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(586, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 589, [586 | __Ss], [__T | __Stack]);
yeccpars2(586, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 588, [586 | __Ss], [__T | __Stack]);
yeccpars2(586, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 587, [586 | __Ss], [__T | __Stack]);
yeccpars2(586, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(587, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  bothway,
 yeccpars2(yeccgoto(topologyDirection, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(588, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  isolate,
 yeccpars2(yeccgoto(topologyDirection, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(589, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  oneway,
 yeccpars2(yeccgoto(topologyDirection, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(590, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'TopologyRequest'{terminationFrom = __1, terminationTo = __3, topologyDirection = __5},
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(topologyTriple, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(591, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [591 | __Ss], [__T | __Stack]);
yeccpars2(591, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(592, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 593, [592 | __Ss], [__T | __Stack]);
yeccpars2(592, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  asn1_NOVALUE,
 yeccpars2(594, __Cat, [592 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(593, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 165, [593 | __Ss], [__T | __Stack]);
yeccpars2(593, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(594, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_commandRequest({subtractReq,__1},#'SubtractRequest'{terminationID = [__3], auditDescriptor = __4}),
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(subtractRequest, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(595, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 596, [595 | __Ss], [__T | __Stack]);
yeccpars2(595, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(596, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __2,
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(optAuditDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(597, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [597 | __Ss], [__T | __Stack]);
yeccpars2(597, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(598, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 599, [598 | __Ss], [__T | __Stack]);
yeccpars2(598, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(599, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 600, [599 | __Ss], [__T | __Stack]);
yeccpars2(599, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(600, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 603, [600 | __Ss], [__T | __Stack]);
yeccpars2(600, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(601, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 602, [601 | __Ss], [__T | __Stack]);
yeccpars2(601, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(602, __Cat, __Ss,  [__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_commandRequest({serviceChangeReq,__1},#'ServiceChangeRequest'{terminationID = [__3], serviceChangeParms = __5}),
 __Nss = lists:nthtail(5, __Ss),
 yeccpars2(yeccgoto(serviceChangeRequest, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(603, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 611, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 610, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 609, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 608, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 607, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 606, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 605, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 604, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 473, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 475, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'DigitMapDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 465, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 474, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 468, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 467, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 469, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 472, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 466, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 470, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 471, [603 | __Ss], [__T | __Stack]);
yeccpars2(603, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(604, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 645, [604 | __Ss], [__T | __Stack]);
yeccpars2(604, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(605, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 643, [605 | __Ss], [__T | __Stack]);
yeccpars2(605, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(606, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 641, [606 | __Ss], [__T | __Stack]);
yeccpars2(606, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(607, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 639, [607 | __Ss], [__T | __Stack]);
yeccpars2(607, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(608, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 637, [608 | __Ss], [__T | __Stack]);
yeccpars2(608, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(609, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 633, [609 | __Ss], [__T | __Stack]);
yeccpars2(609, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(610, 'COMMA', __Ss, [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_timeStamp(__1),
 yeccpars2(yeccgoto(timeStamp, hd(__Ss)), 'COMMA', __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(610, 'RBRKT', __Ss, [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_timeStamp(__1),
 yeccpars2(yeccgoto(timeStamp, hd(__Ss)), 'RBRKT', __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(610, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(611, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 631, [611 | __Ss], [__T | __Stack]);
yeccpars2(611, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(612, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {audit_item,__1},
 yeccpars2(yeccgoto(serviceChangeParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(613, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {extension,__1},
 yeccpars2(yeccgoto(serviceChangeParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(614, 'GREATER', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 214, [614 | __Ss], [__T | __Stack]);
yeccpars2(614, 'LESSER', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 215, [614 | __Ss], [__T | __Stack]);
yeccpars2(614, 'NEQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 216, [614 | __Ss], [__T | __Stack]);
yeccpars2(614, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 213, [614 | __Ss], [__T | __Stack]);
yeccpars2(614, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(615, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_extensionParameter(__1),
 yeccpars2(yeccgoto(extensionParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(616, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {address,__1},
 yeccpars2(yeccgoto(serviceChangeParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(617, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {delay,__1},
 yeccpars2(yeccgoto(serviceChangeParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(618, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {method,__1},
 yeccpars2(yeccgoto(serviceChangeParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(619, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {mgc_id,__1},
 yeccpars2(yeccgoto(serviceChangeParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(620, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 625, [620 | __Ss], [__T | __Stack]);
yeccpars2(620, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(626, __Cat, [620 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(621, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {profile,__1},
 yeccpars2(yeccgoto(serviceChangeParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(622, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {reason,__1},
 yeccpars2(yeccgoto(serviceChangeParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(623, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {version,__1},
 yeccpars2(yeccgoto(serviceChangeParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(624, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {time_stamp,__1},
 yeccpars2(yeccgoto(serviceChangeParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(625, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 611, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 610, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 609, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 608, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 607, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 606, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 605, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 604, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 473, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 475, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'DigitMapDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 465, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 474, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 468, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 467, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 469, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 472, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 466, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 470, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 471, [625 | __Ss], [__T | __Stack]);
yeccpars2(625, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(626, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 627, [626 | __Ss], [__T | __Stack]);
yeccpars2(626, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(627, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_ServiceChangeParm([__3|__4]),
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(serviceChangeDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(628, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 625, [628 | __Ss], [__T | __Stack]);
yeccpars2(628, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(629, __Cat, [628 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(629, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(serviceChangeParms, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(630, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  setelement(#'PropertyParm'.name,__2,__1),
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(extension, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(631, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [631 | __Ss], [__T | __Stack]);
yeccpars2(631, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(632, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_version(__3),
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(serviceChangeVersion, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(633, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'LSBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'LESSER', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, 'SEP', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 1, [633 | __Ss], [__T | __Stack]);
yeccpars2(633, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  no_sep,
 yeccpars2(100, __Cat, [633 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(634, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __3,
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(serviceChangeAddress, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(635, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {portNumber,__3},
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(serviceChangeAddress, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(636, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_uint16(__1),
 yeccpars2(yeccgoto(portNumber, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(637, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, 'QuotedChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 218, [637 | __Ss], [__T | __Stack]);
yeccpars2(637, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(638, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(serviceChangeReason, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(639, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [639 | __Ss], [__T | __Stack]);
yeccpars2(639, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(640, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_profile(__3),
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(serviceChangeProfile, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(641, 'LSBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 96, [641 | __Ss], [__T | __Stack]);
yeccpars2(641, 'LESSER', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 95, [641 | __Ss], [__T | __Stack]);
yeccpars2(641, 'SEP', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 1, [641 | __Ss], [__T | __Stack]);
yeccpars2(641, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  no_sep,
 yeccpars2(100, __Cat, [641 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(642, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __3,
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(serviceChangeMgcId, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(643, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [643 | __Ss], [__T | __Stack]);
yeccpars2(643, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(644, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_serviceChangeMethod(__3),
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(serviceChangeMethod, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(645, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [645 | __Ss], [__T | __Stack]);
yeccpars2(645, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(646, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_uint32(__3),
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(serviceChangeDelay, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(647, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [647 | __Ss], [__T | __Stack]);
yeccpars2(647, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(648, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_uint16(__3),
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(priority, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(649, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [649 | __Ss], [__T | __Stack]);
yeccpars2(649, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(650, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 651, [650 | __Ss], [__T | __Stack]);
yeccpars2(650, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(651, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 652, [651 | __Ss], [__T | __Stack]);
yeccpars2(651, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 108, [651 | __Ss], [__T | __Stack]);
yeccpars2(651, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(652, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 657, [652 | __Ss], [__T | __Stack]);
yeccpars2(652, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(653, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'NotifyRequest'{errorDescriptor = __1},
 yeccpars2(yeccgoto(notifyRequestBody, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(654, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 656, [654 | __Ss], [__T | __Stack]);
yeccpars2(654, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(655, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'NotifyRequest'{observedEventsDescriptor = __1},
 yeccpars2(yeccgoto(notifyRequestBody, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(656, __Cat, __Ss,  [__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_commandRequest({notifyReq,__1},setelement(#'NotifyRequest'.terminationID,__5,[__3])),
 __Nss = lists:nthtail(5, __Ss),
 yeccpars2(yeccgoto(notifyRequest, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(657, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [657 | __Ss], [__T | __Stack]);
yeccpars2(657, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(658, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 659, [658 | __Ss], [__T | __Stack]);
yeccpars2(658, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(659, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 439, [659 | __Ss], [__T | __Stack]);
yeccpars2(659, 'SEP', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 1, [659 | __Ss], [__T | __Stack]);
yeccpars2(659, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  no_sep,
 yeccpars2(442, __Cat, [659 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(660, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 661, [660 | __Ss], [__T | __Stack]);
yeccpars2(660, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(662, __Cat, [660 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(661, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 439, [661 | __Ss], [__T | __Stack]);
yeccpars2(661, 'SEP', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 1, [661 | __Ss], [__T | __Stack]);
yeccpars2(661, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  no_sep,
 yeccpars2(442, __Cat, [661 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(662, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 663, [662 | __Ss], [__T | __Stack]);
yeccpars2(662, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(663, __Cat, __Ss,  [__7,__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'ObservedEventsDescriptor'{requestId = __3, observedEventLst = [__5|__6]},
 __Nss = lists:nthtail(6, __Ss),
 yeccpars2(yeccgoto(observedEventsDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(664, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 661, [664 | __Ss], [__T | __Stack]);
yeccpars2(664, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(665, __Cat, [664 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(665, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(observedEvents, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(666, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 668, [666 | __Ss], [__T | __Stack]);
yeccpars2(666, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 667, [666 | __Ss], [__T | __Stack]);
yeccpars2(666, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 669, [666 | __Ss], [__T | __Stack]);
yeccpars2(666, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(667, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  emergencyAudit,
 yeccpars2(yeccgoto(contextAuditProperty, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(668, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  priorityAudit,
 yeccpars2(yeccgoto(contextAuditProperty, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(669, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  topologyAudit,
 yeccpars2(yeccgoto(contextAuditProperty, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(670, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 671, [670 | __Ss], [__T | __Stack]);
yeccpars2(670, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(672, __Cat, [670 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(671, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 668, [671 | __Ss], [__T | __Stack]);
yeccpars2(671, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 667, [671 | __Ss], [__T | __Stack]);
yeccpars2(671, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 669, [671 | __Ss], [__T | __Stack]);
yeccpars2(671, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(672, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 673, [672 | __Ss], [__T | __Stack]);
yeccpars2(672, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(673, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {contextAudit,[__3|__4]},
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(contextAudit, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(674, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 671, [674 | __Ss], [__T | __Stack]);
yeccpars2(674, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(675, __Cat, [674 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(675, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(contextAuditProperties, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(676, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [676 | __Ss], [__T | __Stack]);
yeccpars2(676, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(677, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 593, [677 | __Ss], [__T | __Stack]);
yeccpars2(677, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  asn1_NOVALUE,
 yeccpars2(678, __Cat, [677 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(678, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_commandRequest({auditValueRequest,__1},#'AuditRequest'{terminationID = __3, auditDescriptor = __4}),
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(auditRequest, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(679, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [679 | __Ss], [__T | __Stack]);
yeccpars2(679, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(680, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 593, [680 | __Ss], [__T | __Stack]);
yeccpars2(680, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  asn1_NOVALUE,
 yeccpars2(681, __Cat, [680 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(681, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_commandRequest({auditCapRequest,__1},#'AuditRequest'{terminationID = __3, auditDescriptor = __4}),
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(auditRequest, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(682, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 124, [682 | __Ss], [__T | __Stack]);
yeccpars2(682, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(683, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_uint32(__1),
 yeccpars2(yeccgoto(transactionID, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(684, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 685, [684 | __Ss], [__T | __Stack]);
yeccpars2(684, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(685, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 124, [685 | __Ss], [__T | __Stack]);
yeccpars2(685, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(686, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 126, [686 | __Ss], [__T | __Stack]);
yeccpars2(686, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(687, __Cat, [686 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(687, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 688, [687 | __Ss], [__T | __Stack]);
yeccpars2(687, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(688, __Cat, __Ss,  [__7,__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'TransactionRequest'{transactionId = ensure_transactionID(__3), actions = [__5|__6]},
 __Nss = lists:nthtail(6, __Ss),
 yeccpars2(yeccgoto(transactionRequest, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(689, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 126, [689 | __Ss], [__T | __Stack]);
yeccpars2(689, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(690, __Cat, [689 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(690, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 691, [690 | __Ss], [__T | __Stack]);
yeccpars2(690, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(691, __Cat, __Ss,  [__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'TransactionRequest'{transactionId = asn1_NOVALUE, actions = [__4|__5]},
 __Nss = lists:nthtail(5, __Ss),
 yeccpars2(yeccgoto(transactionRequest, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(692, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [692 | __Ss], [__T | __Stack]);
yeccpars2(692, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(693, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_transactionAck(__1),
 yeccpars2(yeccgoto(transactionAck, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(694, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 695, [694 | __Ss], [__T | __Stack]);
yeccpars2(694, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(696, __Cat, [694 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(695, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [695 | __Ss], [__T | __Stack]);
yeccpars2(695, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(696, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 697, [696 | __Ss], [__T | __Stack]);
yeccpars2(696, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(697, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__3|__4],
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(transactionResponseAck, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(698, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 695, [698 | __Ss], [__T | __Stack]);
yeccpars2(698, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(699, __Cat, [698 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(699, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(transactionAckList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(700, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [700 | __Ss], [__T | __Stack]);
yeccpars2(700, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(701, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 702, [701 | __Ss], [__T | __Stack]);
yeccpars2(701, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(702, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 703, [702 | __Ss], [__T | __Stack]);
yeccpars2(702, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  asn1_NOVALUE,
 yeccpars2(704, __Cat, [702 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(703, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 841, [703 | __Ss], [__T | __Stack]);
yeccpars2(703, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(704, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 108, [704 | __Ss], [__T | __Stack]);
yeccpars2(704, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 705, [704 | __Ss], [__T | __Stack]);
yeccpars2(704, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(705, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 714, [705 | __Ss], [__T | __Stack]);
yeccpars2(705, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(706, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 710, [706 | __Ss], [__T | __Stack]);
yeccpars2(706, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(711, __Cat, [706 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(707, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {transactionError,__1},
 yeccpars2(yeccgoto(transactionReplyBody, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(708, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 709, [708 | __Ss], [__T | __Stack]);
yeccpars2(708, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(709, __Cat, __Ss,  [__7,__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'TransactionReply'{transactionId = __3, immAckRequired = __5, transactionResult = __6},
 __Nss = lists:nthtail(6, __Ss),
 yeccpars2(yeccgoto(transactionReply, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(710, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 705, [710 | __Ss], [__T | __Stack]);
yeccpars2(710, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(711, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {actionReplies,[__1|__2]},
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(transactionReplyBody, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(712, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 710, [712 | __Ss], [__T | __Stack]);
yeccpars2(712, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(713, __Cat, [712 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(713, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(actionReplyList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(714, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [714 | __Ss], [__T | __Stack]);
yeccpars2(714, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(715, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 716, [715 | __Ss], [__T | __Stack]);
yeccpars2(715, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(716, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 144, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 147, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 108, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 723, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 722, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 718, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 719, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 724, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 720, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 721, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 717, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 139, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 140, [716 | __Ss], [__T | __Stack]);
yeccpars2(716, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(717, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  addReply,
 yeccpars2(yeccgoto(ammsToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(718, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 837, [718 | __Ss], [__T | __Stack]);
yeccpars2(718, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(719, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 817, [719 | __Ss], [__T | __Stack]);
yeccpars2(719, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(720, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  modReply,
 yeccpars2(yeccgoto(ammsToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(721, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  moveReply,
 yeccpars2(yeccgoto(ammsToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(722, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 811, [722 | __Ss], [__T | __Stack]);
yeccpars2(722, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(723, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 786, [723 | __Ss], [__T | __Stack]);
yeccpars2(723, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(724, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  subtractReply,
 yeccpars2(yeccgoto(ammsToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(725, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 785, [725 | __Ss], [__T | __Stack]);
yeccpars2(725, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(726, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {command,__1},
 yeccpars2(yeccgoto(commandReply, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(727, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 739, [727 | __Ss], [__T | __Stack]);
yeccpars2(727, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(728, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {command,__1},
 yeccpars2(yeccgoto(commandReply, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(729, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 734, [729 | __Ss], [__T | __Stack]);
yeccpars2(729, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(735, __Cat, [729 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(730, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {context,__1},
 yeccpars2(yeccgoto(commandReply, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(731, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'ActionReply'{errorDescriptor = __1},
 yeccpars2(yeccgoto(actionReplyBody, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(732, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {command,__1},
 yeccpars2(yeccgoto(commandReply, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(733, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {command,__1},
 yeccpars2(yeccgoto(commandReply, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(734, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 144, [734 | __Ss], [__T | __Stack]);
yeccpars2(734, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 147, [734 | __Ss], [__T | __Stack]);
yeccpars2(734, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 108, [734 | __Ss], [__T | __Stack]);
yeccpars2(734, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 723, [734 | __Ss], [__T | __Stack]);
yeccpars2(734, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 722, [734 | __Ss], [__T | __Stack]);
yeccpars2(734, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 718, [734 | __Ss], [__T | __Stack]);
yeccpars2(734, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 719, [734 | __Ss], [__T | __Stack]);
yeccpars2(734, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 724, [734 | __Ss], [__T | __Stack]);
yeccpars2(734, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 720, [734 | __Ss], [__T | __Stack]);
yeccpars2(734, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 721, [734 | __Ss], [__T | __Stack]);
yeccpars2(734, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 717, [734 | __Ss], [__T | __Stack]);
yeccpars2(734, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 139, [734 | __Ss], [__T | __Stack]);
yeccpars2(734, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 140, [734 | __Ss], [__T | __Stack]);
yeccpars2(734, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(735, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_action_reply([__1|__2]),
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(actionReplyBody, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(736, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 734, [736 | __Ss], [__T | __Stack]);
yeccpars2(736, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(738, __Cat, [736 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(737, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2],
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(commandReplyList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(738, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(commandReplyList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(739, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [739 | __Ss], [__T | __Stack]);
yeccpars2(739, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(740, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 741, [740 | __Ss], [__T | __Stack]);
yeccpars2(740, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  asn1_NOVALUE,
 yeccpars2(742, __Cat, [740 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(741, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 748, [741 | __Ss], [__T | __Stack]);
yeccpars2(741, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 747, [741 | __Ss], [__T | __Stack]);
yeccpars2(741, 'DigitMapDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 166, [741 | __Ss], [__T | __Stack]);
yeccpars2(741, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 744, [741 | __Ss], [__T | __Stack]);
yeccpars2(741, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 746, [741 | __Ss], [__T | __Stack]);
yeccpars2(741, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 172, [741 | __Ss], [__T | __Stack]);
yeccpars2(741, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 168, [741 | __Ss], [__T | __Stack]);
yeccpars2(741, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 745, [741 | __Ss], [__T | __Stack]);
yeccpars2(741, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 167, [741 | __Ss], [__T | __Stack]);
yeccpars2(741, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 743, [741 | __Ss], [__T | __Stack]);
yeccpars2(741, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 108, [741 | __Ss], [__T | __Stack]);
yeccpars2(741, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 466, [741 | __Ss], [__T | __Stack]);
yeccpars2(741, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(742, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {__1,#'AmmsReply'{terminationID = [__3], terminationAudit = __4}},
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(ammsReply, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(743, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 304, [743 | __Ss], [__T | __Stack]);
yeccpars2(743, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  mediaToken,
 yeccpars2(yeccgoto(auditReturnItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(744, 'LSBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 284, [744 | __Ss], [__T | __Stack]);
yeccpars2(744, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 283, [744 | __Ss], [__T | __Stack]);
yeccpars2(744, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  modemToken,
 yeccpars2(yeccgoto(auditReturnItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(745, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 272, [745 | __Ss], [__T | __Stack]);
yeccpars2(745, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  muxToken,
 yeccpars2(yeccgoto(auditReturnItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(746, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 657, [746 | __Ss], [__T | __Stack]);
yeccpars2(746, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  observedEventsToken,
 yeccpars2(yeccgoto(auditReturnItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(747, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 778, [747 | __Ss], [__T | __Stack]);
yeccpars2(747, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  packagesToken,
 yeccpars2(yeccgoto(auditReturnItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(748, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 768, [748 | __Ss], [__T | __Stack]);
yeccpars2(748, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  statsToken,
 yeccpars2(yeccgoto(auditReturnItem, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(749, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {auditReturnItem,__1},
 yeccpars2(yeccgoto(auditReturnParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(750, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 764, [750 | __Ss], [__T | __Stack]);
yeccpars2(750, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(765, __Cat, [750 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(751, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {digitMapDescriptor,__1},
 yeccpars2(yeccgoto(auditReturnParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(752, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {errorDescriptor,__1},
 yeccpars2(yeccgoto(auditReturnParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(753, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {eventBufferDescriptor,__1},
 yeccpars2(yeccgoto(auditReturnParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(754, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {eventsDescriptor,__1},
 yeccpars2(yeccgoto(auditReturnParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(755, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {mediaDescriptor,__1},
 yeccpars2(yeccgoto(auditReturnParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(756, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  '$undefined',
 yeccpars2(yeccgoto(auditReturnParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(757, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {muxDescriptor,__1},
 yeccpars2(yeccgoto(auditReturnParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(758, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {observedEventsDescriptor,__1},
 yeccpars2(yeccgoto(auditReturnParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(759, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {packagesDescriptor,__1},
 yeccpars2(yeccgoto(auditReturnParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(760, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {signalsDescriptor,__1},
 yeccpars2(yeccgoto(auditReturnParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(761, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {statisticsDescriptor,__1},
 yeccpars2(yeccgoto(auditReturnParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(762, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 763, [762 | __Ss], [__T | __Stack]);
yeccpars2(762, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(763, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __2,
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(ammsReplyBody, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(764, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 748, [764 | __Ss], [__T | __Stack]);
yeccpars2(764, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 747, [764 | __Ss], [__T | __Stack]);
yeccpars2(764, 'DigitMapDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 166, [764 | __Ss], [__T | __Stack]);
yeccpars2(764, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 744, [764 | __Ss], [__T | __Stack]);
yeccpars2(764, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 746, [764 | __Ss], [__T | __Stack]);
yeccpars2(764, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 172, [764 | __Ss], [__T | __Stack]);
yeccpars2(764, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 168, [764 | __Ss], [__T | __Stack]);
yeccpars2(764, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 745, [764 | __Ss], [__T | __Stack]);
yeccpars2(764, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 167, [764 | __Ss], [__T | __Stack]);
yeccpars2(764, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 743, [764 | __Ss], [__T | __Stack]);
yeccpars2(764, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 108, [764 | __Ss], [__T | __Stack]);
yeccpars2(764, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 466, [764 | __Ss], [__T | __Stack]);
yeccpars2(764, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(765, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_terminationAudit([__1|__2]),
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(terminationAudit, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(766, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 764, [766 | __Ss], [__T | __Stack]);
yeccpars2(766, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(767, __Cat, [766 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(767, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(auditReturnParameterList, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(768, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [768 | __Ss], [__T | __Stack]);
yeccpars2(768, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(769, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 776, [769 | __Ss], [__T | __Stack]);
yeccpars2(769, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'StatisticsParameter'{statName = __1, statValue = asn1_NOVALUE},
 yeccpars2(yeccgoto(statisticsParameter, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(770, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 771, [770 | __Ss], [__T | __Stack]);
yeccpars2(770, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(772, __Cat, [770 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(771, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [771 | __Ss], [__T | __Stack]);
yeccpars2(771, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(772, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 773, [772 | __Ss], [__T | __Stack]);
yeccpars2(772, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(773, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__3|__4],
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(statisticsDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(774, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 771, [774 | __Ss], [__T | __Stack]);
yeccpars2(774, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(775, __Cat, [774 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(775, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(statisticsParameters, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(776, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, 'QuotedChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 218, [776 | __Ss], [__T | __Stack]);
yeccpars2(776, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(777, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'StatisticsParameter'{statName = __1, statValue = [__3]},
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(statisticsParameter, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(778, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [778 | __Ss], [__T | __Stack]);
yeccpars2(778, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(779, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 780, [779 | __Ss], [__T | __Stack]);
yeccpars2(779, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(781, __Cat, [779 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(780, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [780 | __Ss], [__T | __Stack]);
yeccpars2(780, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(781, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 782, [781 | __Ss], [__T | __Stack]);
yeccpars2(781, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(782, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__3|__4],
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(packagesDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(783, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 780, [783 | __Ss], [__T | __Stack]);
yeccpars2(783, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(784, __Cat, [783 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(784, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(packagesItems, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(785, __Cat, __Ss,  [__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  setelement(#'ActionReply'.contextId,__5,__3),
 __Nss = lists:nthtail(5, __Ss),
 yeccpars2(yeccgoto(actionReply, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(786, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [786 | __Ss], [__T | __Stack]);
yeccpars2(786, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(787, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 788, [787 | __Ss], [__T | __Stack]);
yeccpars2(787, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  {serviceChangeResParms,#'ServiceChangeResParm'{}},
 yeccpars2(789, __Cat, [787 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(788, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 790, [788 | __Ss], [__T | __Stack]);
yeccpars2(788, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 108, [788 | __Ss], [__T | __Stack]);
yeccpars2(788, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(789, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {serviceChangeReply,#'ServiceChangeReply'{terminationID = [__3], serviceChangeResult = __4}},
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(serviceChangeReply, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(790, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 795, [790 | __Ss], [__T | __Stack]);
yeccpars2(790, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(791, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 794, [791 | __Ss], [__T | __Stack]);
yeccpars2(791, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(792, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 793, [792 | __Ss], [__T | __Stack]);
yeccpars2(792, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(793, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {serviceChangeResParms,__2},
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(serviceChangeReplyBody, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(794, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {errorDescriptor,__2},
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(serviceChangeReplyBody, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(795, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 439, [795 | __Ss], [__T | __Stack]);
yeccpars2(795, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 799, [795 | __Ss], [__T | __Stack]);
yeccpars2(795, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 797, [795 | __Ss], [__T | __Stack]);
yeccpars2(795, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 796, [795 | __Ss], [__T | __Stack]);
yeccpars2(795, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 798, [795 | __Ss], [__T | __Stack]);
yeccpars2(795, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(796, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 641, [796 | __Ss], [__T | __Stack]);
yeccpars2(796, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(797, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 639, [797 | __Ss], [__T | __Stack]);
yeccpars2(797, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(798, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 633, [798 | __Ss], [__T | __Stack]);
yeccpars2(798, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(799, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 631, [799 | __Ss], [__T | __Stack]);
yeccpars2(799, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(800, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 806, [800 | __Ss], [__T | __Stack]);
yeccpars2(800, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(807, __Cat, [800 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(801, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {address,__1},
 yeccpars2(yeccgoto(servChgReplyParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(802, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {mgc_id,__1},
 yeccpars2(yeccgoto(servChgReplyParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(803, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {profile,__1},
 yeccpars2(yeccgoto(servChgReplyParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(804, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {version,__1},
 yeccpars2(yeccgoto(servChgReplyParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(805, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {time_stamp,__1},
 yeccpars2(yeccgoto(servChgReplyParm, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(806, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 439, [806 | __Ss], [__T | __Stack]);
yeccpars2(806, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 799, [806 | __Ss], [__T | __Stack]);
yeccpars2(806, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 797, [806 | __Ss], [__T | __Stack]);
yeccpars2(806, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 796, [806 | __Ss], [__T | __Stack]);
yeccpars2(806, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 798, [806 | __Ss], [__T | __Stack]);
yeccpars2(806, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(807, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 808, [807 | __Ss], [__T | __Stack]);
yeccpars2(807, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(808, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  merge_ServiceChangeResParm([__3|__4]),
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(serviceChangeReplyDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(809, 'COMMA', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 806, [809 | __Ss], [__T | __Stack]);
yeccpars2(809, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(810, __Cat, [809 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(810, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__2|__3],
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(servChgReplyParms, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(811, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [811 | __Ss], [__T | __Stack]);
yeccpars2(811, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(812, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 813, [812 | __Ss], [__T | __Stack]);
yeccpars2(812, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  asn1_NOVALUE,
 yeccpars2(814, __Cat, [812 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(813, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 108, [813 | __Ss], [__T | __Stack]);
yeccpars2(813, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(814, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {notifyReply,#'NotifyReply'{terminationID = [__3], errorDescriptor = __4}},
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(notifyReply, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(815, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 816, [815 | __Ss], [__T | __Stack]);
yeccpars2(815, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(816, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  __2,
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(notifyReplyBody, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(817, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 818, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [817 | __Ss], [__T | __Stack]);
yeccpars2(817, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(818, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 824, [818 | __Ss], [__T | __Stack]);
yeccpars2(818, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(819, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {auditValueReply,__3},
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(auditReply, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(820, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 821, [820 | __Ss], [__T | __Stack]);
yeccpars2(820, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {auditResult,#'AuditResult'{terminationID = __1, terminationAuditResult = []}},
 yeccpars2(yeccgoto(auditOther, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(821, 'StatsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 748, [821 | __Ss], [__T | __Stack]);
yeccpars2(821, 'PackagesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 747, [821 | __Ss], [__T | __Stack]);
yeccpars2(821, 'DigitMapDescriptorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 166, [821 | __Ss], [__T | __Stack]);
yeccpars2(821, 'ModemToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 744, [821 | __Ss], [__T | __Stack]);
yeccpars2(821, 'ObservedEventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 746, [821 | __Ss], [__T | __Stack]);
yeccpars2(821, 'SignalsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 172, [821 | __Ss], [__T | __Stack]);
yeccpars2(821, 'EventsToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 168, [821 | __Ss], [__T | __Stack]);
yeccpars2(821, 'MuxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 745, [821 | __Ss], [__T | __Stack]);
yeccpars2(821, 'EventBufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 167, [821 | __Ss], [__T | __Stack]);
yeccpars2(821, 'MediaToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 743, [821 | __Ss], [__T | __Stack]);
yeccpars2(821, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 108, [821 | __Ss], [__T | __Stack]);
yeccpars2(821, 'DigitMapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 466, [821 | __Ss], [__T | __Stack]);
yeccpars2(821, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(822, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 823, [822 | __Ss], [__T | __Stack]);
yeccpars2(822, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(823, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {auditResult,#'AuditResult'{terminationID = __1, terminationAuditResult = __3}},
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(auditOther, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(824, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 827, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [824 | __Ss], [__T | __Stack]);
yeccpars2(824, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(825, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {auditValueReply,__4},
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(auditReply, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(826, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {contextAuditResult,__1},
 yeccpars2(yeccgoto(contextTerminationAudit, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(827, 'EQUAL', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 830, [827 | __Ss], [__T | __Stack]);
yeccpars2(827, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(828, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 829, [828 | __Ss], [__T | __Stack]);
yeccpars2(828, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(829, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {contextAuditResult,__2},
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(contextTerminationAudit, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(830, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [830 | __Ss], [__T | __Stack]);
yeccpars2(830, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(831, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 833, [831 | __Ss], [__T | __Stack]);
yeccpars2(831, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(832, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_uint(__1,0,999),
 yeccpars2(yeccgoto(errorCode, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(833, 'QuotedChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 834, [833 | __Ss], [__T | __Stack]);
yeccpars2(833, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  asn1_NOVALUE,
 yeccpars2(835, __Cat, [833 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(834, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  value_of(__1),
 yeccpars2(yeccgoto(errorText, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(835, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 836, [835 | __Ss], [__T | __Stack]);
yeccpars2(835, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(836, __Cat, __Ss,  [__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'ErrorDescriptor'{errorCode = __3, errorText = __5},
 __Nss = lists:nthtail(5, __Ss),
 yeccpars2(yeccgoto(errorDescriptor, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(837, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 838, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [837 | __Ss], [__T | __Stack]);
yeccpars2(837, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(838, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 824, [838 | __Ss], [__T | __Stack]);
yeccpars2(838, __Cat, __Ss,  [__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  make_safe_token(__1),
 yeccpars2(yeccgoto(safeToken, hd(__Ss)), __Cat, __Ss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(839, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {auditCapReply,__3},
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(auditReply, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(840, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  {auditCapReply,__4},
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(auditReply, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(841, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  'NULL',
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(optImmAckRequired, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(842, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [842 | __Ss], [__T | __Stack]);
yeccpars2(842, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(843, 'LBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 844, [843 | __Ss], [__T | __Stack]);
yeccpars2(843, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(844, 'RBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 845, [844 | __Ss], [__T | __Stack]);
yeccpars2(844, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(845, __Cat, __Ss,  [__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'TransactionPending'{transactionId = ensure_transactionID(__3)},
 __Nss = lists:nthtail(4, __Ss),
 yeccpars2(yeccgoto(transactionPending, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(846, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, 'COLON', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 846, [846 | __Ss], [__T | __Stack]);
yeccpars2(846, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(854, __Cat, [846 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(847, 'RSBRKT', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 850, [847 | __Ss], [__T | __Stack]);
yeccpars2(847, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(848, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, 'COLON', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 846, [848 | __Ss], [__T | __Stack]);
yeccpars2(848, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  [],
 yeccpars2(849, __Cat, [848 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(849, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [__1|__2],
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(daddr, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(850, 'COLON', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 851, [850 | __Ss], [__T | __Stack]);
yeccpars2(850, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_domainAddress(__2,asn1_NOVALUE),
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(domainAddress, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(851, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [851 | __Ss], [__T | __Stack]);
yeccpars2(851, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(852, 'SEP', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 1, [852 | __Ss], [__T | __Stack]);
yeccpars2(852, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  no_sep,
 yeccpars2(853, __Cat, [852 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(853, __Cat, __Ss,  [__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_domainAddress(__2,__5),
 __Nss = lists:nthtail(5, __Ss),
 yeccpars2(yeccgoto(domainAddress, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(854, __Cat, __Ss,  [__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  [colon|__2],
 __Nss = lists:nthtail(1, __Ss),
 yeccpars2(yeccgoto(daddr, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(855, 'GREATER', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 856, [855 | __Ss], [__T | __Stack]);
yeccpars2(855, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(856, 'COLON', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 857, [856 | __Ss], [__T | __Stack]);
yeccpars2(856, __Cat, __Ss,  [__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_domainName(__2,asn1_NOVALUE),
 __Nss = lists:nthtail(2, __Ss),
 yeccpars2(yeccgoto(domainName, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(857, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [857 | __Ss], [__T | __Stack]);
yeccpars2(857, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(858, 'SEP', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 1, [858 | __Ss], [__T | __Stack]);
yeccpars2(858, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  no_sep,
 yeccpars2(859, __Cat, [858 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(859, __Cat, __Ss,  [__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_domainName(__2,__5),
 __Nss = lists:nthtail(5, __Ss),
 yeccpars2(yeccgoto(domainName, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(860, __Cat, __Ss,  [__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  #'MegacoMessage'{authHeader = __2, mess = __3},
 __Nss = lists:nthtail(3, __Ss),
 yeccpars2(yeccgoto(megacoMessage, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(861, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [861 | __Ss], [__T | __Stack]);
yeccpars2(861, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(862, 'COLON', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 863, [862 | __Ss], [__T | __Stack]);
yeccpars2(862, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(863, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [863 | __Ss], [__T | __Stack]);
yeccpars2(863, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(864, 'COLON', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 865, [864 | __Ss], [__T | __Stack]);
yeccpars2(864, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(865, 'VersionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 92, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'V91Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 91, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'V90Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 90, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'V76Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 89, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'V34Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 88, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'V32bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 87, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'V32Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 86, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'V22bisToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 85, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'V22Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 84, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'V18Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 83, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'TransToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 82, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'TopologyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 81, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'TimeStampToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 80, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'TimeOutToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 79, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'TestToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 78, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'TerminationStateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 77, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'SynchISDNToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 76, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'SubtractToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 75, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'StreamToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 74, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'SignalTypeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 73, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'SignalListToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 72, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'ServiceChangeAddressToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 68, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'ServiceChangeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 69, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'ServiceStatesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 70, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'ServicesToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 71, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'SendrecvToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 67, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'SendonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 66, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'ReservedValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 62, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'ReservedGroupToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 61, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'RestartToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 64, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'ResponseAckToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 63, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'ReplyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 60, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'RecvonlyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 59, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'ReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 58, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'ProfileToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 57, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'PriorityToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 56, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'PendingToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 55, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'OtherReasonToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 53, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'OutOfSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 54, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'OnOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 50, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'OnToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 51, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'OffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 49, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'OnewayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 52, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'Nx64Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 48, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'NotifyCompletionToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 46, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'NotifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 47, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'MoveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 45, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'ModifyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 44, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'ModeToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 43, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'MgcIdToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 42, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'MethodToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 41, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'LockStepToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 39, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'LoopbackToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 40, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'LocalControlToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 38, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'KeepActiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 37, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'InSvcToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 32, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'IsolateToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 36, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'InterruptByNewSignalsDescrToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 35, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'InterruptByEventToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 34, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'InactiveToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 33, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'ImmAckRequiredToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 31, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'HandOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 30, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'H226Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 29, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'H223Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 28, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'H221Token', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 27, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'GracefulToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 26, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'ForcedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 25, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'FailoverToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 24, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'ErrorToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 23, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'EmergencyOffToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 21, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'EmergencyToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 22, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'EmbedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 20, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'DurationToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 19, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'DelayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 16, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'DisconnectedToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 18, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'DiscardToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 17, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'ContextAuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 14, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'CtxToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 15, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'BufferToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 13, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'BriefToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 12, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'BothwayToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 11, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'AuthToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 10, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'AuditValueToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 9, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'AuditCapToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 7, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'AuditToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 8, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'AddToken', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 6, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, 'SafeChars', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 65, [865 | __Ss], [__T | __Stack]);
yeccpars2(865, _, _, _, __T, _, _) ->
 yeccerror(__T);
yeccpars2(866, 'SEP', __Ss, __Stack, __T, __Ts, __Tzr) ->
 yeccpars1(__Ts, __Tzr, 1, [866 | __Ss], [__T | __Stack]);
yeccpars2(866, __Cat, __Ss,  __Stack, __T, __Ts, __Tzr) ->
 __Val =  no_sep,
 yeccpars2(867, __Cat, [866 | __Ss], [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(867, __Cat, __Ss,  [__8,__7,__6,__5,__4,__3,__2,__1|__Stack], __T, __Ts, __Tzr) ->
 __Val =  ensure_auth_header(__3,__5,__7),
 __Nss = lists:nthtail(7, __Ss),
 yeccpars2(yeccgoto(authenticationHeader, hd(__Nss)), __Cat, __Nss, [__Val | __Stack], __T, __Ts, __Tzr);
yeccpars2(__Other, _, _, _, _, _, _) ->
 exit({parser, __Other, missing_state_in_action_table}).

yeccgoto(actionReply, 704) ->
 706;
yeccgoto(actionReply, 710) ->
 712;
yeccgoto(actionReplyBody, 716) ->
 725;
yeccgoto(actionReplyList, 706) ->
 711;
yeccgoto(actionReplyList, 712) ->
 713;
yeccgoto(actionRequest, 123) ->
 125;
yeccgoto(actionRequest, 126) ->
 129;
yeccgoto(actionRequest, 682) ->
 689;
yeccgoto(actionRequest, 685) ->
 686;
yeccgoto(actionRequestItem, 134) ->
 148;
yeccgoto(actionRequestItem, 569) ->
 572;
yeccgoto(actionRequestItems, 148) ->
 570;
yeccgoto(actionRequestItems, 572) ->
 573;
yeccgoto(actionRequestList, 125) ->
 127;
yeccgoto(actionRequestList, 129) ->
 130;
yeccgoto(actionRequestList, 686) ->
 687;
yeccgoto(actionRequestList, 689) ->
 690;
yeccgoto(alternativeValue, 213) ->
 225;
yeccgoto(ammParameter, 163) ->
 173;
yeccgoto(ammParameter, 182) ->
 185;
yeccgoto(ammParameters, 173) ->
 183;
yeccgoto(ammParameters, 185) ->
 186;
yeccgoto(ammRequest, 134) ->
 149;
yeccgoto(ammRequest, 569) ->
 149;
yeccgoto(ammRequestBody, 162) ->
 164;
yeccgoto(ammToken, 134) ->
 150;
yeccgoto(ammToken, 569) ->
 150;
yeccgoto(ammsReply, 716) ->
 726;
yeccgoto(ammsReply, 734) ->
 726;
yeccgoto(ammsReplyBody, 740) ->
 742;
yeccgoto(ammsToken, 716) ->
 727;
yeccgoto(ammsToken, 734) ->
 727;
yeccgoto(auditDescriptor, 163) ->
 174;
yeccgoto(auditDescriptor, 182) ->
 174;
yeccgoto(auditDescriptor, 593) ->
 595;
yeccgoto(auditDescriptorBody, 464) ->
 476;
yeccgoto(auditItem, 464) ->
 477;
yeccgoto(auditItem, 564) ->
 566;
yeccgoto(auditItem, 603) ->
 612;
yeccgoto(auditItem, 625) ->
 612;
yeccgoto(auditItemList, 477) ->
 565;
yeccgoto(auditItemList, 566) ->
 567;
yeccgoto(auditOther, 817) ->
 819;
yeccgoto(auditOther, 837) ->
 839;
yeccgoto(auditReply, 716) ->
 728;
yeccgoto(auditReply, 734) ->
 728;
yeccgoto(auditRequest, 134) ->
 151;
yeccgoto(auditRequest, 569) ->
 151;
yeccgoto(auditReturnItem, 464) ->
 478;
yeccgoto(auditReturnItem, 564) ->
 478;
yeccgoto(auditReturnItem, 603) ->
 478;
yeccgoto(auditReturnItem, 625) ->
 478;
yeccgoto(auditReturnItem, 741) ->
 749;
yeccgoto(auditReturnItem, 764) ->
 749;
yeccgoto(auditReturnItem, 821) ->
 749;
yeccgoto(auditReturnParameter, 741) ->
 750;
yeccgoto(auditReturnParameter, 764) ->
 766;
yeccgoto(auditReturnParameter, 821) ->
 750;
yeccgoto(auditReturnParameterList, 750) ->
 765;
yeccgoto(auditReturnParameterList, 766) ->
 767;
yeccgoto(authenticationHeader, 3) ->
 5;
yeccgoto(commandReply, 716) ->
 729;
yeccgoto(commandReply, 734) ->
 736;
yeccgoto(commandReplyList, 729) ->
 735;
yeccgoto(commandReplyList, 736) ->
 738;
yeccgoto(commandRequest, 134) ->
 152;
yeccgoto(commandRequest, 569) ->
 152;
yeccgoto(contextAudit, 134) ->
 153;
yeccgoto(contextAudit, 569) ->
 153;
yeccgoto(contextAuditProperties, 670) ->
 672;
yeccgoto(contextAuditProperties, 674) ->
 675;
yeccgoto(contextAuditProperty, 666) ->
 670;
yeccgoto(contextAuditProperty, 671) ->
 674;
yeccgoto(contextID, 131) ->
 132;
yeccgoto(contextID, 714) ->
 715;
yeccgoto(contextProperty, 134) ->
 154;
yeccgoto(contextProperty, 569) ->
 154;
yeccgoto(contextProperty, 716) ->
 730;
yeccgoto(contextProperty, 734) ->
 730;
yeccgoto(contextTerminationAudit, 818) ->
 825;
yeccgoto(contextTerminationAudit, 838) ->
 840;
yeccgoto(daddr, 96) ->
 847;
yeccgoto(daddr, 846) ->
 854;
yeccgoto(daddr, 848) ->
 849;
yeccgoto(deviceName, 100) ->
 102;
yeccgoto(digitMapDescriptor, 163) ->
 175;
yeccgoto(digitMapDescriptor, 182) ->
 175;
yeccgoto(digitMapDescriptor, 741) ->
 751;
yeccgoto(digitMapDescriptor, 764) ->
 751;
yeccgoto(digitMapDescriptor, 821) ->
 751;
yeccgoto(domainAddress, 94) ->
 97;
yeccgoto(domainAddress, 633) ->
 97;
yeccgoto(domainAddress, 641) ->
 97;
yeccgoto(domainName, 94) ->
 98;
yeccgoto(domainName, 633) ->
 98;
yeccgoto(domainName, 641) ->
 98;
yeccgoto(embedFirst, 403) ->
 405;
yeccgoto(embedFirst, 407) ->
 409;
yeccgoto(embedNoSig, 385) ->
 390;
yeccgoto(embedNoSig, 398) ->
 390;
yeccgoto(embedSig, 422) ->
 426;
yeccgoto(embedSig, 430) ->
 426;
yeccgoto(embedWithSig, 385) ->
 391;
yeccgoto(embedWithSig, 398) ->
 391;
yeccgoto(errorCode, 830) ->
 831;
yeccgoto(errorDescriptor, 99) ->
 113;
yeccgoto(errorDescriptor, 651) ->
 653;
yeccgoto(errorDescriptor, 704) ->
 707;
yeccgoto(errorDescriptor, 716) ->
 731;
yeccgoto(errorDescriptor, 734) ->
 737;
yeccgoto(errorDescriptor, 741) ->
 752;
yeccgoto(errorDescriptor, 764) ->
 752;
yeccgoto(errorDescriptor, 788) ->
 791;
yeccgoto(errorDescriptor, 813) ->
 815;
yeccgoto(errorDescriptor, 821) ->
 752;
yeccgoto(errorDescriptor, 824) ->
 828;
yeccgoto(errorText, 833) ->
 835;
yeccgoto(eventBufferControl, 320) ->
 323;
yeccgoto(eventBufferControl, 327) ->
 323;
yeccgoto(eventBufferControlState, 337) ->
 340;
yeccgoto(eventBufferDescriptor, 163) ->
 176;
yeccgoto(eventBufferDescriptor, 182) ->
 176;
yeccgoto(eventBufferDescriptor, 741) ->
 753;
yeccgoto(eventBufferDescriptor, 764) ->
 753;
yeccgoto(eventBufferDescriptor, 821) ->
 753;
yeccgoto(eventDM, 385) ->
 392;
yeccgoto(eventDM, 398) ->
 392;
yeccgoto(eventDM, 422) ->
 427;
yeccgoto(eventDM, 430) ->
 427;
yeccgoto(eventParameter, 385) ->
 393;
yeccgoto(eventParameter, 398) ->
 401;
yeccgoto(eventParameterName, 385) ->
 394;
yeccgoto(eventParameterName, 398) ->
 394;
yeccgoto(eventParameterName, 422) ->
 394;
yeccgoto(eventParameterName, 430) ->
 394;
yeccgoto(eventParameterName, 448) ->
 394;
yeccgoto(eventParameterName, 452) ->
 394;
yeccgoto(eventParameterName, 554) ->
 557;
yeccgoto(eventParameters, 393) ->
 399;
yeccgoto(eventParameters, 401) ->
 402;
yeccgoto(eventSpec, 438) ->
 440;
yeccgoto(eventSpec, 459) ->
 462;
yeccgoto(eventSpecList, 440) ->
 460;
yeccgoto(eventSpecList, 462) ->
 463;
yeccgoto(eventStream, 554) ->
 558;
yeccgoto(eventStreamOrOther, 385) ->
 395;
yeccgoto(eventStreamOrOther, 398) ->
 395;
yeccgoto(eventStreamOrOther, 422) ->
 428;
yeccgoto(eventStreamOrOther, 430) ->
 428;
yeccgoto(eventStreamOrOther, 448) ->
 450;
yeccgoto(eventStreamOrOther, 452) ->
 450;
yeccgoto(eventsDescriptor, 163) ->
 177;
yeccgoto(eventsDescriptor, 182) ->
 177;
yeccgoto(eventsDescriptor, 741) ->
 754;
yeccgoto(eventsDescriptor, 764) ->
 754;
yeccgoto(eventsDescriptor, 821) ->
 754;
yeccgoto(extension, 603) ->
 613;
yeccgoto(extension, 625) ->
 613;
yeccgoto(extensionParameter, 603) ->
 614;
yeccgoto(extensionParameter, 625) ->
 614;
yeccgoto(indAudauditReturnParameter, 464) ->
 479;
yeccgoto(indAudauditReturnParameter, 488) ->
 496;
yeccgoto(indAudauditReturnParameter, 564) ->
 479;
yeccgoto(indAudauditReturnParameter, 603) ->
 479;
yeccgoto(indAudauditReturnParameter, 625) ->
 479;
yeccgoto(indAuddigitMapDescriptor, 464) ->
 480;
yeccgoto(indAuddigitMapDescriptor, 488) ->
 480;
yeccgoto(indAuddigitMapDescriptor, 564) ->
 480;
yeccgoto(indAuddigitMapDescriptor, 603) ->
 480;
yeccgoto(indAuddigitMapDescriptor, 625) ->
 480;
yeccgoto(indAudeventBufferDescriptor, 464) ->
 481;
yeccgoto(indAudeventBufferDescriptor, 488) ->
 481;
yeccgoto(indAudeventBufferDescriptor, 564) ->
 481;
yeccgoto(indAudeventBufferDescriptor, 603) ->
 481;
yeccgoto(indAudeventBufferDescriptor, 625) ->
 481;
yeccgoto(indAudeventSpec, 551) ->
 552;
yeccgoto(indAudeventSpecParameter, 554) ->
 559;
yeccgoto(indAudeventsDescriptor, 464) ->
 482;
yeccgoto(indAudeventsDescriptor, 488) ->
 482;
yeccgoto(indAudeventsDescriptor, 564) ->
 482;
yeccgoto(indAudeventsDescriptor, 603) ->
 482;
yeccgoto(indAudeventsDescriptor, 625) ->
 482;
yeccgoto(indAudlocalControlDescriptor, 518) ->
 522;
yeccgoto(indAudlocalControlDescriptor, 534) ->
 522;
yeccgoto(indAudlocalParm, 537) ->
 538;
yeccgoto(indAudlocalParm, 540) ->
 543;
yeccgoto(indAudlocalParmList, 538) ->
 541;
yeccgoto(indAudlocalParmList, 543) ->
 544;
yeccgoto(indAudmediaDescriptor, 464) ->
 483;
yeccgoto(indAudmediaDescriptor, 488) ->
 483;
yeccgoto(indAudmediaDescriptor, 564) ->
 483;
yeccgoto(indAudmediaDescriptor, 603) ->
 483;
yeccgoto(indAudmediaDescriptor, 625) ->
 483;
yeccgoto(indAudmediaParm, 518) ->
 523;
yeccgoto(indAudpackagesDescriptor, 464) ->
 484;
yeccgoto(indAudpackagesDescriptor, 488) ->
 484;
yeccgoto(indAudpackagesDescriptor, 564) ->
 484;
yeccgoto(indAudpackagesDescriptor, 603) ->
 484;
yeccgoto(indAudpackagesDescriptor, 625) ->
 484;
yeccgoto(indAudrequestedEvent, 547) ->
 548;
yeccgoto(indAudsignalList, 501) ->
 505;
yeccgoto(indAudsignalParm, 501) ->
 506;
yeccgoto(indAudsignalsDescriptor, 464) ->
 485;
yeccgoto(indAudsignalsDescriptor, 488) ->
 485;
yeccgoto(indAudsignalsDescriptor, 564) ->
 485;
yeccgoto(indAudsignalsDescriptor, 603) ->
 485;
yeccgoto(indAudsignalsDescriptor, 625) ->
 485;
yeccgoto(indAudstatisticsDescriptor, 464) ->
 486;
yeccgoto(indAudstatisticsDescriptor, 488) ->
 486;
yeccgoto(indAudstatisticsDescriptor, 564) ->
 486;
yeccgoto(indAudstatisticsDescriptor, 603) ->
 486;
yeccgoto(indAudstatisticsDescriptor, 625) ->
 486;
yeccgoto(indAudstreamDescriptor, 518) ->
 524;
yeccgoto(indAudstreamParm, 518) ->
 525;
yeccgoto(indAudstreamParm, 534) ->
 535;
yeccgoto(indAudterminationAudit, 464) ->
 487;
yeccgoto(indAudterminationAudit, 564) ->
 487;
yeccgoto(indAudterminationAudit, 603) ->
 487;
yeccgoto(indAudterminationAudit, 625) ->
 487;
yeccgoto(indAudterminationAuditList, 479) ->
 489;
yeccgoto(indAudterminationAuditList, 496) ->
 497;
yeccgoto(indAudterminationStateDescriptor, 518) ->
 526;
yeccgoto(indAudterminationStateParm, 528) ->
 529;
yeccgoto(localControlDescriptor, 304) ->
 310;
yeccgoto(localControlDescriptor, 315) ->
 310;
yeccgoto(localControlDescriptor, 343) ->
 310;
yeccgoto(localControlDescriptor, 345) ->
 310;
yeccgoto(localParm, 350) ->
 354;
yeccgoto(localParm, 356) ->
 359;
yeccgoto(localParmList, 354) ->
 357;
yeccgoto(localParmList, 359) ->
 360;
yeccgoto(mId, 94) ->
 99;
yeccgoto(mId, 633) ->
 634;
yeccgoto(mId, 641) ->
 642;
yeccgoto(mediaDescriptor, 163) ->
 178;
yeccgoto(mediaDescriptor, 182) ->
 178;
yeccgoto(mediaDescriptor, 741) ->
 755;
yeccgoto(mediaDescriptor, 764) ->
 755;
yeccgoto(mediaDescriptor, 821) ->
 755;
yeccgoto(mediaParm, 304) ->
 311;
yeccgoto(mediaParm, 315) ->
 318;
yeccgoto(mediaParmList, 311) ->
 316;
yeccgoto(mediaParmList, 318) ->
 319;
yeccgoto(megacoMessage, 0) ->
 2;
yeccgoto(message, 5) ->
 93;
yeccgoto(messageBody, 99) ->
 114;
yeccgoto(modemDescriptor, 163) ->
 179;
yeccgoto(modemDescriptor, 182) ->
 179;
yeccgoto(modemDescriptor, 741) ->
 756;
yeccgoto(modemDescriptor, 764) ->
 756;
yeccgoto(modemDescriptor, 821) ->
 756;
yeccgoto(modemType, 283) ->
 302;
yeccgoto(modemType, 284) ->
 285;
yeccgoto(modemType, 287) ->
 300;
yeccgoto(modemTypeList, 285) ->
 288;
yeccgoto(modemTypeList, 300) ->
 301;
yeccgoto(mtpAddress, 100) ->
 103;
yeccgoto(muxDescriptor, 163) ->
 180;
yeccgoto(muxDescriptor, 182) ->
 180;
yeccgoto(muxDescriptor, 741) ->
 757;
yeccgoto(muxDescriptor, 764) ->
 757;
yeccgoto(muxDescriptor, 821) ->
 757;
yeccgoto(muxType, 272) ->
 273;
yeccgoto(notificationReason, 248) ->
 253;
yeccgoto(notificationReason, 254) ->
 257;
yeccgoto(notificationReasons, 253) ->
 255;
yeccgoto(notificationReasons, 257) ->
 258;
yeccgoto(notifyReply, 716) ->
 732;
yeccgoto(notifyReply, 734) ->
 732;
yeccgoto(notifyReplyBody, 812) ->
 814;
yeccgoto(notifyRequest, 134) ->
 155;
yeccgoto(notifyRequest, 569) ->
 155;
yeccgoto(notifyRequestBody, 651) ->
 654;
yeccgoto(observedEvent, 438) ->
 441;
yeccgoto(observedEvent, 459) ->
 441;
yeccgoto(observedEvent, 659) ->
 660;
yeccgoto(observedEvent, 661) ->
 664;
yeccgoto(observedEventBody, 447) ->
 449;
yeccgoto(observedEventBody, 457) ->
 458;
yeccgoto(observedEventParameter, 448) ->
 451;
yeccgoto(observedEventParameter, 452) ->
 455;
yeccgoto(observedEventParameters, 451) ->
 453;
yeccgoto(observedEventParameters, 455) ->
 456;
yeccgoto(observedEvents, 660) ->
 662;
yeccgoto(observedEvents, 664) ->
 665;
yeccgoto(observedEventsDescriptor, 651) ->
 655;
yeccgoto(observedEventsDescriptor, 741) ->
 758;
yeccgoto(observedEventsDescriptor, 764) ->
 758;
yeccgoto(observedEventsDescriptor, 821) ->
 758;
yeccgoto(onOrOff, 361) ->
 364;
yeccgoto(onOrOff, 365) ->
 366;
yeccgoto(optAuditDescriptor, 592) ->
 594;
yeccgoto(optAuditDescriptor, 677) ->
 678;
yeccgoto(optAuditDescriptor, 680) ->
 681;
yeccgoto(optImmAckRequired, 702) ->
 704;
yeccgoto(optIndAudeventSpecParameter, 553) ->
 555;
yeccgoto(optIndAudsignalParm, 474) ->
 502;
yeccgoto(optIndAudsignalParm, 494) ->
 502;
yeccgoto(optPropertyParms, 289) ->
 291;
yeccgoto(optPropertyParms, 302) ->
 303;
yeccgoto(optSep, 0) ->
 3;
yeccgoto(optSep, 94) ->
 100;
yeccgoto(optSep, 102) ->
 107;
yeccgoto(optSep, 103) ->
 106;
yeccgoto(optSep, 438) ->
 442;
yeccgoto(optSep, 443) ->
 444;
yeccgoto(optSep, 445) ->
 446;
yeccgoto(optSep, 459) ->
 442;
yeccgoto(optSep, 633) ->
 100;
yeccgoto(optSep, 641) ->
 100;
yeccgoto(optSep, 659) ->
 442;
yeccgoto(optSep, 661) ->
 442;
yeccgoto(optSep, 852) ->
 853;
yeccgoto(optSep, 858) ->
 859;
yeccgoto(optSep, 866) ->
 867;
yeccgoto(packagesDescriptor, 741) ->
 759;
yeccgoto(packagesDescriptor, 764) ->
 759;
yeccgoto(packagesDescriptor, 821) ->
 759;
yeccgoto(packagesItem, 514) ->
 515;
yeccgoto(packagesItem, 778) ->
 779;
yeccgoto(packagesItem, 780) ->
 783;
yeccgoto(packagesItems, 779) ->
 781;
yeccgoto(packagesItems, 783) ->
 784;
yeccgoto(parmValue, 206) ->
 217;
yeccgoto(parmValue, 292) ->
 299;
yeccgoto(parmValue, 394) ->
 397;
yeccgoto(parmValue, 614) ->
 630;
yeccgoto(pathName, 100) ->
 104;
yeccgoto(pkgdName, 187) ->
 189;
yeccgoto(pkgdName, 195) ->
 189;
yeccgoto(pkgdName, 264) ->
 189;
yeccgoto(pkgdName, 267) ->
 189;
yeccgoto(pkgdName, 290) ->
 292;
yeccgoto(pkgdName, 294) ->
 292;
yeccgoto(pkgdName, 320) ->
 292;
yeccgoto(pkgdName, 327) ->
 292;
yeccgoto(pkgdName, 350) ->
 292;
yeccgoto(pkgdName, 356) ->
 292;
yeccgoto(pkgdName, 377) ->
 378;
yeccgoto(pkgdName, 380) ->
 378;
yeccgoto(pkgdName, 414) ->
 415;
yeccgoto(pkgdName, 417) ->
 415;
yeccgoto(pkgdName, 442) ->
 457;
yeccgoto(pkgdName, 446) ->
 447;
yeccgoto(pkgdName, 498) ->
 499;
yeccgoto(pkgdName, 501) ->
 189;
yeccgoto(pkgdName, 511) ->
 189;
yeccgoto(pkgdName, 547) ->
 549;
yeccgoto(pkgdName, 551) ->
 553;
yeccgoto(pkgdName, 768) ->
 769;
yeccgoto(pkgdName, 771) ->
 769;
yeccgoto(portNumber, 633) ->
 635;
yeccgoto(portNumber, 851) ->
 852;
yeccgoto(portNumber, 857) ->
 858;
yeccgoto(priority, 134) ->
 156;
yeccgoto(priority, 569) ->
 156;
yeccgoto(priority, 716) ->
 156;
yeccgoto(priority, 734) ->
 156;
yeccgoto(propertyParm, 290) ->
 293;
yeccgoto(propertyParm, 294) ->
 297;
yeccgoto(propertyParm, 320) ->
 324;
yeccgoto(propertyParm, 327) ->
 324;
yeccgoto(propertyParm, 350) ->
 355;
yeccgoto(propertyParm, 356) ->
 355;
yeccgoto(propertyParms, 293) ->
 295;
yeccgoto(propertyParms, 297) ->
 298;
yeccgoto(requestID, 374) ->
 375;
yeccgoto(requestID, 412) ->
 413;
yeccgoto(requestID, 545) ->
 546;
yeccgoto(requestID, 657) ->
 658;
yeccgoto(requestedEvent, 377) ->
 379;
yeccgoto(requestedEvent, 380) ->
 383;
yeccgoto(requestedEventBody, 378) ->
 386;
yeccgoto(requestedEvents, 379) ->
 381;
yeccgoto(requestedEvents, 383) ->
 384;
yeccgoto(safeToken, 5) ->
 94;
yeccgoto(safeToken, 95) ->
 855;
yeccgoto(safeToken, 96) ->
 848;
yeccgoto(safeToken, 100) ->
 105;
yeccgoto(safeToken, 122) ->
 683;
yeccgoto(safeToken, 131) ->
 133;
yeccgoto(safeToken, 160) ->
 161;
yeccgoto(safeToken, 187) ->
 190;
yeccgoto(safeToken, 195) ->
 190;
yeccgoto(safeToken, 200) ->
 206;
yeccgoto(safeToken, 208) ->
 206;
yeccgoto(safeToken, 213) ->
 219;
yeccgoto(safeToken, 214) ->
 219;
yeccgoto(safeToken, 215) ->
 219;
yeccgoto(safeToken, 216) ->
 219;
yeccgoto(safeToken, 223) ->
 219;
yeccgoto(safeToken, 224) ->
 219;
yeccgoto(safeToken, 228) ->
 219;
yeccgoto(safeToken, 229) ->
 219;
yeccgoto(safeToken, 239) ->
 240;
yeccgoto(safeToken, 259) ->
 260;
yeccgoto(safeToken, 261) ->
 262;
yeccgoto(safeToken, 264) ->
 190;
yeccgoto(safeToken, 267) ->
 190;
yeccgoto(safeToken, 272) ->
 274;
yeccgoto(safeToken, 275) ->
 161;
yeccgoto(safeToken, 278) ->
 161;
yeccgoto(safeToken, 283) ->
 286;
yeccgoto(safeToken, 284) ->
 286;
yeccgoto(safeToken, 287) ->
 286;
yeccgoto(safeToken, 290) ->
 190;
yeccgoto(safeToken, 294) ->
 190;
yeccgoto(safeToken, 320) ->
 190;
yeccgoto(safeToken, 327) ->
 190;
yeccgoto(safeToken, 341) ->
 240;
yeccgoto(safeToken, 350) ->
 190;
yeccgoto(safeToken, 356) ->
 190;
yeccgoto(safeToken, 374) ->
 376;
yeccgoto(safeToken, 377) ->
 190;
yeccgoto(safeToken, 380) ->
 190;
yeccgoto(safeToken, 385) ->
 396;
yeccgoto(safeToken, 398) ->
 396;
yeccgoto(safeToken, 412) ->
 376;
yeccgoto(safeToken, 414) ->
 190;
yeccgoto(safeToken, 417) ->
 190;
yeccgoto(safeToken, 422) ->
 396;
yeccgoto(safeToken, 430) ->
 396;
yeccgoto(safeToken, 442) ->
 190;
yeccgoto(safeToken, 446) ->
 190;
yeccgoto(safeToken, 448) ->
 396;
yeccgoto(safeToken, 452) ->
 396;
yeccgoto(safeToken, 498) ->
 190;
yeccgoto(safeToken, 501) ->
 190;
yeccgoto(safeToken, 509) ->
 262;
yeccgoto(safeToken, 511) ->
 190;
yeccgoto(safeToken, 514) ->
 516;
yeccgoto(safeToken, 528) ->
 530;
yeccgoto(safeToken, 532) ->
 240;
yeccgoto(safeToken, 537) ->
 539;
yeccgoto(safeToken, 540) ->
 539;
yeccgoto(safeToken, 545) ->
 376;
yeccgoto(safeToken, 547) ->
 190;
yeccgoto(safeToken, 551) ->
 190;
yeccgoto(safeToken, 554) ->
 396;
yeccgoto(safeToken, 561) ->
 240;
yeccgoto(safeToken, 574) ->
 161;
yeccgoto(safeToken, 578) ->
 161;
yeccgoto(safeToken, 583) ->
 161;
yeccgoto(safeToken, 591) ->
 161;
yeccgoto(safeToken, 597) ->
 161;
yeccgoto(safeToken, 603) ->
 615;
yeccgoto(safeToken, 625) ->
 615;
yeccgoto(safeToken, 631) ->
 632;
yeccgoto(safeToken, 633) ->
 636;
yeccgoto(safeToken, 637) ->
 219;
yeccgoto(safeToken, 639) ->
 640;
yeccgoto(safeToken, 643) ->
 644;
yeccgoto(safeToken, 645) ->
 646;
yeccgoto(safeToken, 647) ->
 648;
yeccgoto(safeToken, 649) ->
 161;
yeccgoto(safeToken, 657) ->
 376;
yeccgoto(safeToken, 676) ->
 161;
yeccgoto(safeToken, 679) ->
 161;
yeccgoto(safeToken, 692) ->
 693;
yeccgoto(safeToken, 695) ->
 693;
yeccgoto(safeToken, 700) ->
 683;
yeccgoto(safeToken, 714) ->
 133;
yeccgoto(safeToken, 739) ->
 161;
yeccgoto(safeToken, 768) ->
 190;
yeccgoto(safeToken, 771) ->
 190;
yeccgoto(safeToken, 776) ->
 219;
yeccgoto(safeToken, 778) ->
 516;
yeccgoto(safeToken, 780) ->
 516;
yeccgoto(safeToken, 786) ->
 161;
yeccgoto(safeToken, 811) ->
 161;
yeccgoto(safeToken, 817) ->
 161;
yeccgoto(safeToken, 824) ->
 161;
yeccgoto(safeToken, 830) ->
 832;
yeccgoto(safeToken, 837) ->
 161;
yeccgoto(safeToken, 842) ->
 683;
yeccgoto(safeToken, 846) ->
 848;
yeccgoto(safeToken, 848) ->
 848;
yeccgoto(safeToken, 851) ->
 636;
yeccgoto(safeToken, 857) ->
 636;
yeccgoto(safeToken, 861) ->
 862;
yeccgoto(safeToken, 863) ->
 864;
yeccgoto(safeToken, 865) ->
 866;
yeccgoto(secondEventParameter, 422) ->
 429;
yeccgoto(secondEventParameter, 430) ->
 433;
yeccgoto(secondEventParameters, 429) ->
 431;
yeccgoto(secondEventParameters, 433) ->
 434;
yeccgoto(secondRequestedEvent, 414) ->
 416;
yeccgoto(secondRequestedEvent, 417) ->
 420;
yeccgoto(secondRequestedEventBody, 415) ->
 423;
yeccgoto(secondRequestedEvents, 416) ->
 418;
yeccgoto(secondRequestedEvents, 420) ->
 421;
yeccgoto(servChgReplyParm, 795) ->
 800;
yeccgoto(servChgReplyParm, 806) ->
 809;
yeccgoto(servChgReplyParms, 800) ->
 807;
yeccgoto(servChgReplyParms, 809) ->
 810;
yeccgoto(serviceChangeAddress, 603) ->
 616;
yeccgoto(serviceChangeAddress, 625) ->
 616;
yeccgoto(serviceChangeAddress, 795) ->
 801;
yeccgoto(serviceChangeAddress, 806) ->
 801;
yeccgoto(serviceChangeDelay, 603) ->
 617;
yeccgoto(serviceChangeDelay, 625) ->
 617;
yeccgoto(serviceChangeDescriptor, 599) ->
 601;
yeccgoto(serviceChangeMethod, 603) ->
 618;
yeccgoto(serviceChangeMethod, 625) ->
 618;
yeccgoto(serviceChangeMgcId, 603) ->
 619;
yeccgoto(serviceChangeMgcId, 625) ->
 619;
yeccgoto(serviceChangeMgcId, 795) ->
 802;
yeccgoto(serviceChangeMgcId, 806) ->
 802;
yeccgoto(serviceChangeParm, 603) ->
 620;
yeccgoto(serviceChangeParm, 625) ->
 628;
yeccgoto(serviceChangeParms, 620) ->
 626;
yeccgoto(serviceChangeParms, 628) ->
 629;
yeccgoto(serviceChangeProfile, 603) ->
 621;
yeccgoto(serviceChangeProfile, 625) ->
 621;
yeccgoto(serviceChangeProfile, 795) ->
 803;
yeccgoto(serviceChangeProfile, 806) ->
 803;
yeccgoto(serviceChangeReason, 603) ->
 622;
yeccgoto(serviceChangeReason, 625) ->
 622;
yeccgoto(serviceChangeReply, 716) ->
 733;
yeccgoto(serviceChangeReply, 734) ->
 733;
yeccgoto(serviceChangeReplyBody, 787) ->
 789;
yeccgoto(serviceChangeReplyDescriptor, 788) ->
 792;
yeccgoto(serviceChangeRequest, 134) ->
 157;
yeccgoto(serviceChangeRequest, 569) ->
 157;
yeccgoto(serviceChangeVersion, 603) ->
 623;
yeccgoto(serviceChangeVersion, 625) ->
 623;
yeccgoto(serviceChangeVersion, 795) ->
 804;
yeccgoto(serviceChangeVersion, 806) ->
 804;
yeccgoto(serviceState, 332) ->
 336;
yeccgoto(serviceStates, 320) ->
 325;
yeccgoto(serviceStates, 327) ->
 325;
yeccgoto(sigParameter, 200) ->
 207;
yeccgoto(sigParameter, 208) ->
 211;
yeccgoto(sigParameters, 207) ->
 209;
yeccgoto(sigParameters, 211) ->
 212;
yeccgoto(signalList, 187) ->
 191;
yeccgoto(signalList, 195) ->
 191;
yeccgoto(signalListId, 261) ->
 263;
yeccgoto(signalListId, 509) ->
 510;
yeccgoto(signalListParm, 264) ->
 265;
yeccgoto(signalListParm, 267) ->
 270;
yeccgoto(signalListParm, 511) ->
 512;
yeccgoto(signalListParms, 265) ->
 268;
yeccgoto(signalListParms, 270) ->
 271;
yeccgoto(signalName, 187) ->
 192;
yeccgoto(signalName, 195) ->
 192;
yeccgoto(signalName, 264) ->
 192;
yeccgoto(signalName, 267) ->
 192;
yeccgoto(signalName, 501) ->
 192;
yeccgoto(signalName, 511) ->
 192;
yeccgoto(signalParm, 187) ->
 193;
yeccgoto(signalParm, 195) ->
 198;
yeccgoto(signalParms, 193) ->
 196;
yeccgoto(signalParms, 198) ->
 199;
yeccgoto(signalRequest, 187) ->
 194;
yeccgoto(signalRequest, 195) ->
 194;
yeccgoto(signalRequest, 264) ->
 266;
yeccgoto(signalRequest, 267) ->
 266;
yeccgoto(signalRequest, 501) ->
 507;
yeccgoto(signalRequest, 511) ->
 266;
yeccgoto(signalType, 242) ->
 246;
yeccgoto(signalsDescriptor, 163) ->
 181;
yeccgoto(signalsDescriptor, 182) ->
 181;
yeccgoto(signalsDescriptor, 403) ->
 406;
yeccgoto(signalsDescriptor, 435) ->
 436;
yeccgoto(signalsDescriptor, 741) ->
 760;
yeccgoto(signalsDescriptor, 764) ->
 760;
yeccgoto(signalsDescriptor, 821) ->
 760;
yeccgoto(statisticsDescriptor, 741) ->
 761;
yeccgoto(statisticsDescriptor, 764) ->
 761;
yeccgoto(statisticsDescriptor, 821) ->
 761;
yeccgoto(statisticsParameter, 768) ->
 770;
yeccgoto(statisticsParameter, 771) ->
 774;
yeccgoto(statisticsParameters, 770) ->
 772;
yeccgoto(statisticsParameters, 774) ->
 775;
yeccgoto(streamDescriptor, 304) ->
 312;
yeccgoto(streamDescriptor, 315) ->
 312;
yeccgoto(streamID, 239) ->
 241;
yeccgoto(streamID, 341) ->
 342;
yeccgoto(streamID, 532) ->
 533;
yeccgoto(streamID, 561) ->
 562;
yeccgoto(streamModes, 367) ->
 373;
yeccgoto(streamParm, 304) ->
 313;
yeccgoto(streamParm, 315) ->
 313;
yeccgoto(streamParm, 343) ->
 344;
yeccgoto(streamParm, 345) ->
 348;
yeccgoto(streamParmList, 344) ->
 346;
yeccgoto(streamParmList, 348) ->
 349;
yeccgoto(subtractRequest, 134) ->
 158;
yeccgoto(subtractRequest, 569) ->
 158;
yeccgoto(terminationA, 574) ->
 575;
yeccgoto(terminationA, 578) ->
 575;
yeccgoto(terminationAudit, 741) ->
 762;
yeccgoto(terminationAudit, 821) ->
 822;
yeccgoto(terminationB, 583) ->
 584;
yeccgoto(terminationID, 160) ->
 162;
yeccgoto(terminationID, 275) ->
 277;
yeccgoto(terminationID, 278) ->
 281;
yeccgoto(terminationID, 574) ->
 576;
yeccgoto(terminationID, 578) ->
 576;
yeccgoto(terminationID, 583) ->
 585;
yeccgoto(terminationID, 591) ->
 592;
yeccgoto(terminationID, 597) ->
 598;
yeccgoto(terminationID, 649) ->
 650;
yeccgoto(terminationID, 676) ->
 677;
yeccgoto(terminationID, 679) ->
 680;
yeccgoto(terminationID, 739) ->
 740;
yeccgoto(terminationID, 786) ->
 787;
yeccgoto(terminationID, 811) ->
 812;
yeccgoto(terminationID, 817) ->
 820;
yeccgoto(terminationID, 824) ->
 277;
yeccgoto(terminationID, 837) ->
 820;
yeccgoto(terminationIDList, 273) ->
 276;
yeccgoto(terminationIDList, 818) ->
 826;
yeccgoto(terminationIDList, 838) ->
 826;
yeccgoto(terminationIDListRepeat, 277) ->
 279;
yeccgoto(terminationIDListRepeat, 281) ->
 282;
yeccgoto(terminationStateDescriptor, 304) ->
 314;
yeccgoto(terminationStateDescriptor, 315) ->
 314;
yeccgoto(terminationStateParm, 320) ->
 326;
yeccgoto(terminationStateParm, 327) ->
 330;
yeccgoto(terminationStateParms, 326) ->
 328;
yeccgoto(terminationStateParms, 330) ->
 331;
yeccgoto(timeStamp, 438) ->
 443;
yeccgoto(timeStamp, 459) ->
 443;
yeccgoto(timeStamp, 603) ->
 624;
yeccgoto(timeStamp, 625) ->
 624;
yeccgoto(timeStamp, 659) ->
 443;
yeccgoto(timeStamp, 661) ->
 443;
yeccgoto(timeStamp, 795) ->
 805;
yeccgoto(timeStamp, 806) ->
 805;
yeccgoto(topologyDescriptor, 134) ->
 159;
yeccgoto(topologyDescriptor, 569) ->
 159;
yeccgoto(topologyDescriptor, 716) ->
 159;
yeccgoto(topologyDescriptor, 734) ->
 159;
yeccgoto(topologyDirection, 586) ->
 590;
yeccgoto(topologyTriple, 574) ->
 577;
yeccgoto(topologyTriple, 578) ->
 581;
yeccgoto(topologyTripleList, 577) ->
 579;
yeccgoto(topologyTripleList, 581) ->
 582;
yeccgoto(transactionAck, 692) ->
 694;
yeccgoto(transactionAck, 695) ->
 698;
yeccgoto(transactionAckList, 694) ->
 696;
yeccgoto(transactionAckList, 698) ->
 699;
yeccgoto(transactionID, 122) ->
 684;
yeccgoto(transactionID, 700) ->
 701;
yeccgoto(transactionID, 842) ->
 843;
yeccgoto(transactionItem, 99) ->
 115;
yeccgoto(transactionItem, 115) ->
 115;
yeccgoto(transactionList, 99) ->
 116;
yeccgoto(transactionList, 115) ->
 121;
yeccgoto(transactionPending, 99) ->
 117;
yeccgoto(transactionPending, 115) ->
 117;
yeccgoto(transactionReply, 99) ->
 118;
yeccgoto(transactionReply, 115) ->
 118;
yeccgoto(transactionReplyBody, 704) ->
 708;
yeccgoto(transactionRequest, 99) ->
 119;
yeccgoto(transactionRequest, 115) ->
 119;
yeccgoto(transactionResponseAck, 99) ->
 120;
yeccgoto(transactionResponseAck, 115) ->
 120;
yeccgoto(value, 213) ->
 226;
yeccgoto(value, 214) ->
 222;
yeccgoto(value, 215) ->
 221;
yeccgoto(value, 216) ->
 220;
yeccgoto(value, 223) ->
 236;
yeccgoto(value, 224) ->
 227;
yeccgoto(value, 228) ->
 234;
yeccgoto(value, 229) ->
 232;
yeccgoto(value, 637) ->
 638;
yeccgoto(value, 776) ->
 777;
yeccgoto(valueList, 227) ->
 230;
yeccgoto(valueList, 232) ->
 233;
yeccgoto(valueList, 236) ->
 237;
yeccgoto(__Symbol, __State) ->
 exit({__Symbol, __State, missing_in_goto_table}).


